//
//  Consultes.swift
//  Concot
//
//  Created by Andreu Font Bardolet on 8/5/15.
//  Copyright (c) 2015 Andreu Font Bardolet. All rights reserved.
//

/*
Aquest document conté funcions bàsiques primaries útilitzades general al obrir l'aplicació
Permeten comprovar si hi ha vehicles, i obtenir-los
*/

import Foundation
import UIKit
import CoreData

class Consultes {
    
    private func obtenirContext() -> NSManagedObjectContext {
        //Definició de la conexió a la BDD usada a la resta de la classe
        let appDelegate:AppDelegate = (UIApplication.sharedApplication().delegate as! AppDelegate)
        return appDelegate.managedObjectContext!
    }
    
    func obtenirVehicle(nom: String) -> Vehicle? {
        //Creem el Context
        let context = obtenirContext()
        
        //Creem la consulta
        let fetchRequest = NSFetchRequest()
        
        //Indiquem l'entitat a consultar
        let vehicle = NSEntityDescription.entityForName("Vehicle", inManagedObjectContext:context)
        fetchRequest.entity = vehicle
        
        //Filtre/condicional usant predicats
        let predicate = NSPredicate(format: "%K = %@", "nom", (nom as NSObject))
        fetchRequest.predicate = predicate
        
        //Fer la consulta
        let resultat = (context.executeFetchRequest(fetchRequest, error: nil)!) as! [Vehicle]
        
        //LLegim si hi ha resultats i retornem una resposta
        if resultat.count > 0 {
            let retornar:Vehicle = resultat.first! as Vehicle
            return retornar
        } else {
            return nil
        }
    }
    
    func obtenirPrimerVehicle() -> Vehicle? {
        //Creem el Context
        let context = obtenirContext()
        
        //Creem la consulta
        let fetchRequest = NSFetchRequest()
        
        //Indiquem l'entitat a consultar
        let vehicle = NSEntityDescription.entityForName("Vehicle", inManagedObjectContext:context)
        fetchRequest.entity = vehicle
        
        //nº de resultats que volem obtenir en mode Batch, per límitar el nº de resultats és el fetchLimit
        //fetchRequest.fetchBatchSize = 1
        
        //Limitem a 1 resultat
        fetchRequest.fetchLimit = 1
        
        //Cadena d'ordenació dels resultats
        let sortDescriptor = NSSortDescriptor(key: "nom", ascending: false)
        let sortDescriptors = [sortDescriptor]
        fetchRequest.sortDescriptors = [sortDescriptor]
        
        //Fem la consulta
        let resultat = (context.executeFetchRequest(fetchRequest, error: nil)!) as! [Vehicle]
        
        //LLegim si hi ha resultats i retornem una resposta
        if resultat.count > 0 {
            let retornar:Vehicle = resultat.first! as Vehicle
            return retornar
        } else {
            return nil
        }
    }
    
    func hiHaVehicles() -> Bool {
        //Creem el Context
        let context = obtenirContext()
        
        //Creem la consulta
        let fetchRequest = NSFetchRequest()
        
        //Indiquem l'entitat a consultar
        let vehicle = NSEntityDescription.entityForName("Vehicle", inManagedObjectContext:context)
        fetchRequest.entity = vehicle
        
        //Fem la consulta
        let resultat = (context.executeFetchRequest(fetchRequest, error: nil)!) as! [Vehicle]
        
        //LLegim si hi ha resultats i retornem una resposta
        if resultat.count > 0 {
            return true
        } else {
            return false
        }
    }
    
}