//
//  Extensions.swift
//  Concot
//
//  Created by Andreu Font Bardolet on 7/5/15.
//  Copyright (c) 2015 Andreu Font Bardolet. All rights reserved.
//

/*
En aquest document es troben les funcions i variables calculades que s'afegeixen com a ampliació a l'entitat de la base de dades "Emplenament"
Aquestes permeten obtenir propietats extra (calculades) i funcions per realitzar operacions amb aquesta entitat
*/

import Foundation

extension Emplenament {
    
    func guardarDataEmplenament(entrada: String) {
        //Funció que permet guardar la data de realització de l'emplenament a partir d'un string
        self.guardarData(entrada)
    }
    
    //MARK: - Propietats calculades
    
    var emplenamentsAnteriors: [Emplenament] {
        //retorna la llista d'emplenaments anteriors
        get {
            return self.vehicle.obtenirEmplenaments(false, quilometres: self.contaquilometres)
        }
    }
    
    var emplenamentsSeguents: [Emplenament] {
        //Retorna la llista d'emplenaments següents
        get {
            return self.vehicle.obtenirEmplenaments(true, quilometres: self.contaquilometres)
        }
    }
    
    var contaquilometres: Double {
        //Mostra ja formatat a double els quilometres del vehicle al fer l'emplenament
        get{
            return Double(self.km)
        }
    }
    
    var quantitat: Double {
        //Mostra na en double la quantitat posada
        get {
            return Double(self.litres)
       }
    }
    
    var pleBool: Bool {
        //Mostra en Bool la si s'ha fet diposit ple
        get {
            return Bool(self.ple)
        }
    }
    
    var preuDouble: Double {
        //Mostra en double el preu del carburant
        get {
            return Double(self.preu)
        }
    }

    var dataString: String {
        //Mostra en string la data de l'emplenament
        get {
            return self.obtenirData()
        }
    }
    
    var monedaDet: Moneda {
        //Retorna l'objecte moneda de la moneda usada
        get {
            return Moneda(string: self.moneda)
        }
    }
    
    var unitatDistancia: Distancia {
        //Retorna l'objecte distància per saber la unitat de distància
        get {
            return Distancia(string: self.vehicle.unitatKm)
        }
    }
    
    var distanciaFeta: Double {
        //Retorna els quilòmetres que s'han fet amb aquest dipòsit des de l'anterior emplenament (útil per calcular el consum)
        get {
            if self.anterior != nil {
                return self.contaquilometres - self.anterior!.contaquilometres
            } else {
                return 0
            }
        }
    }
    
    var distanciaAconseguida: Double {
        //Retonra la distància que s'ha fet amb aquest emplenament de diposit fins al seguent emplenament
        get {
            if self.seguent != nil {
                return self.seguent!.contaquilometres - self.contaquilometres
            } else {
                return self.vehicle.contaquilometres - self.contaquilometres
            }
        }
    }
    
    var consumCada100Distancia: Double? {
        //Retorna el consum de carburant cada 100 km respecte el darrer emplenament complet i només si aquest diposit també és un emplenament complet
        get {
            var retorn: Double? = nil
            if (self.pleBool && (self.seguent?.pleBool ?? false )) {
                retorn = (seguent!.quantitat/self.distanciaAconseguida) * 100.0
            } else if self.pleBool && self.seguent != nil {
                let emplenaments = self.emplenamentsSeguents
                //Sumar distancia i carburant acumulats des de l'últim emplenament complet
                var distanciaAcumulada = self.distanciaAconseguida
                var litresAcumulats = 0.0//self.quantitat
                var continuar = true
                var index = 0
                do {
                    if Bool(emplenaments[index].ple) {
                        continuar = false
                        litresAcumulats += emplenaments[index].quantitat
                    } else {
                        distanciaAcumulada += emplenaments[index].distanciaAconseguida
                        litresAcumulats += emplenaments[index].quantitat
                    }
                    index += 1
                } while continuar && index < emplenaments.count
                if continuar != true {
                    //Hem arribat a trobar un emplenament complet i per tant es pot fer el càlcul
                    retorn = (litresAcumulats/distanciaAcumulada) * 100.0
                }
            }
            return retorn
        }
    }
    
    var consumDistancia: Double? {
        //Retorna el consum de carburant respecte el darrer emplenament complet i només si aquest diposit també és un emplenament complet
        get {
            if self.consumCada100Distancia != nil {
                return self.consumCada100Distancia!/100.0
            } else {
                return nil
            }
        }
    }
    
    var costCada100Distancia: Double? {
        //Retorna el cost de carburant cada 100 km respecte el darrer emplenament complet i només si aquest diposit també és un emplenament complet
        get {
            if self.costDistancia != nil {
                return self.costDistancia!*100.0
            } else {
                return nil
            }
        }
    }
    
    var costDistancia: Double? {
        //Retorna el cost de carburant respecte el darrer emplenament complet i només si aquest diposit també és un emplenament complet
        get {
            var retorn: Double? = nil
            if (self.pleBool && (self.seguent?.pleBool ?? false )) {
                retorn = seguent!.costDouble/self.distanciaAconseguida
            } else  if self.pleBool && self.seguent != nil {
                let emplenaments = self.emplenamentsSeguents
                //Sumar distancia i cost acumulats des de l'últim emplenament complet
                var distanciaAcumulada = self.distanciaAconseguida
                var costAcumulat = 0.0//self.costDouble
                var continuar = true
                var index = 0
                do {
                    if Bool(emplenaments[index].ple) {
                        continuar = false
                        costAcumulat += emplenaments[index].costDouble
                    } else {
                        distanciaAcumulada += emplenaments[index].distanciaAconseguida
                        costAcumulat += emplenaments[index].costDouble
                    }
                    index += 1
                } while continuar && index < emplenaments.count
                if continuar != true {
                    //Hem arribat a trobar un emplenament complet i per tant es pot fer el càlcul
                    retorn = costAcumulat/distanciaAcumulada
                }
            }
            return retorn
        }
    }
}