//
//  ExtensioManteniment.swift
//  Concot
//
//  Created by Andreu Font Bardolet on 8/6/15.
//  Copyright (c) 2015 Andreu Font Bardolet. All rights reserved.
//

/*
Aquest document afegeix funcions i propietats calculades a l'entitat Manteniment de la base de dades
*/

import Foundation

extension Manteniment {
    
    func obtenirRegistre() -> [RegistreManteniment] {
        //Busca i retorna els registres d'aquest manteniment i vehicle
        return self.vehicle.obtenirRegistresDeMantenimentDe(self.nom)
    }
    
    func obtenirUltimManteniment() -> RegistreManteniment? {
        //Busca el darrer registre que hi ha d'aquest manteniment i vehicle
        var retorn: RegistreManteniment? = nil
        
        let manteniments = self.vehicle.obtenirRegistresDeMantenimentDe(self.nom)
        if manteniments.count > 0 {
            let ordre = NSComparisonResult.OrderedDescending
            retorn = manteniments.sorted({ $0.data.compare($1.data) == ordre}).first
        }
        return retorn
    }
    
    var desgast: Double? {
        get{
            // Valor que va de 0 a 1 (o més gran que 1 si està sobre-desgastat) 0 = nou, 0<1 = nivell de desgast, 1 = gastat, >1 = sobre-desgastat!
            let manteniments = self.vehicle.obtenirRegistresDeMantenimentDe(self.nom)
            var retorn: Double? = nil
            if manteniments.count > 0 {
                //calculem del desgast segons distancia recorreguda
                if manteniments[0].distanciaMaxima != nil {
                    if manteniments[0].distanciaMuntatgeFinal == nil{
                        let distanciaFeta = self.vehicle.contaquilometres - Double(manteniments[0].distanciaMuntatgeInicial)
                        //Obtenim el nivell de desgast d'un valor 0=nou fins a 1=gastat
                       retorn = distanciaFeta / (Double(manteniments[0].distanciaMaxima!) + Double(manteniments[0].distanciaMuntatgeInicial))
                    } else {
                        let distanciaFeta = Double(manteniments[0].distanciaMuntatgeFinal!) - Double(manteniments[0].distanciaMuntatgeInicial)
                        //Obtenim el nivell de desgast d'un valor 0=nou fins a 1=gastat
                        retorn = distanciaFeta / (Double(manteniments[0].distanciaMaxima!) + Double(manteniments[0].distanciaMuntatgeInicial))
                    }
                }
                //calculem el desgast segons el factor temps
                if manteniments[0].tempsCaducitat != nil {
                    var desgastTemps = 0.0
                    
                    let avui = NSDate()
                    let calendari = NSCalendar.currentCalendar()
                    let intervalTemps = NSDateComponents()
                    intervalTemps.month = Int(manteniments[0].tempsCaducitat!)
                    let caducaElDia = calendari.dateByAddingComponents(intervalTemps, toDate: manteniments[0].data, options: nil)!
                    
                    if manteniments[0].dataCanvi == nil {
                        let data = NSDate()
                        //let diesRestants = caducaElDia.daysFrom(data)
                        let diesFets = data.daysFrom(manteniments[0].data)
                        let diesMax = caducaElDia.daysFrom(manteniments[0].data)
                        desgastTemps = Double(diesFets) / Double(diesMax)
                    } else {
                        let diesFets = manteniments[0].dataCanvi!.daysFrom(caducaElDia)
                        let diesMax = manteniments[0].data.daysFrom(caducaElDia)
                        desgastTemps = Double(diesFets) / Double(diesMax)
                    }
                    if retorn < desgastTemps {
                        retorn = desgastTemps
                    }
                }
            }
            return retorn
        }
    }
    
    var costPerUnitatDistancia: Double? {
        //Mostra el cost per quilòmetre en desgast segons el cost del darrer registre i la informació dels quilòmetres que pot recorrer la peca fins el proper canvi
        get{
            let manteniments = self.vehicle.obtenirRegistresDeMantenimentDe(self.nom)
            if manteniments.count > 0 {
                if manteniments[0].distanciaMaxima != nil {
                    if manteniments[0].distanciaMuntatgeFinal == nil{
                        return Double(manteniments[0].cost) / Double(manteniments[0].distanciaMaxima!)
                    } else {
                        let distanciaFeta = Double(manteniments[0].distanciaMuntatgeFinal!) - Double(manteniments[0].distanciaMuntatgeInicial)
                        return Double(manteniments[0].cost) / distanciaFeta
                    }
                } else {
                    return nil
                }
            } else {
                return nil
            }
        }
    }
    
    var dataActual: NSDate? {
        //Retorna la data del darrer manteniment
        get {
            if let darrer = self.obtenirUltimManteniment() {
                return darrer.data
            }
            return nil
        }
    }

    var properCanvi: Double? {
        //Mostra quans quilòmetres tindrà el vehicle al proper canvi
        get{
            let manteniments = self.vehicle.obtenirRegistresDeMantenimentDe(self.nom)
            if manteniments.count > 0 {
                if manteniments[0].distanciaMaxima != nil {
                    if manteniments[0].distanciaMuntatgeFinal == nil{
                        return Double(manteniments[0].distanciaMuntatgeInicial) + Double(manteniments[0].distanciaMaxima!)
                    } else {
                        return nil
                    }
                } else {
                    return nil
                }
            } else {
                return nil
            }
        }
    }
    
    var dataProperCanvi: NSDate? {
        //Mostra a quina data cal fer el proper canvi
        get{
            let manteniments = self.vehicle.obtenirRegistresDeMantenimentDe(self.nom)
            if manteniments.count > 0 {
                if manteniments[0].tempsCaducitat != nil {
                    let avui = NSDate()
                    let calendari = NSCalendar.currentCalendar()
                    let intervalTemps = NSDateComponents()
                    intervalTemps.month = Int(manteniments[0].tempsCaducitat!)
                    return calendari.dateByAddingComponents(intervalTemps, toDate: manteniments[0].data, options: nil)!
                } else {
                    return nil
                }
            }
            return nil
        }
    }
    
    var tempsRestant: Int? {
        //Mostra els mesos que falten per el proper canvi
        return self.dataProperCanvi?.monthsFrom(NSDate())

    }
    
    var distanciaRestant: Double? {
        //Mostra els quilòmetres que es poden recorrer abans del proper canvi
        get {
            if self.properCanvi != nil {
                return self.properCanvi! - self.vehicle.contaquilometres
            } else {
                return nil
            }
        }
    }
}