//
//  GraphView.swift
//  Concot
//
//  Created by Andreu Font Bardolet on 13/5/15.
//  Copyright (c) 2015 Andreu Font Bardolet. All rights reserved.
//

import UIKit


@IBDesignable class GraphView: UIView {
    
    //Propoietats editables des de l'story board
    @IBInspectable var colorInicial: UIColor = UIColor.darkGrayColor()
    @IBInspectable var colorFinal: UIColor = UIColor.blackColor()
    @IBInspectable var cantosRodons: Bool = false
    @IBInspectable var radiCantos: Double = 8.0
    @IBInspectable var margeEsquerra: CGFloat = 30
    @IBInspectable var margeDret: CGFloat = 10
    @IBInspectable var margeSuperior: CGFloat = 50
    @IBInspectable var margeInferior: CGFloat = 30
    @IBInspectable var invertir: Bool = false
    
    var graphPoints:[Double] = [2, 4, 3, 6, 7, 9, 11, 9, 10, 11]
    
    override func drawRect(rect: CGRect) {
        
        //calculem/obtenim variables estàtques
        let width = rect.width
        let height = rect.height
        let margeTotal = self.margeEsquerra + self.margeDret
        
        //let topBorder:CGFloat = 50
        //let bottomBorder:CGFloat = 30
        let graphHeight = height - self.margeSuperior - self.margeInferior
        
        //Arrodonir cantos del requadre del gràfic
        if self.cantosRodons{
            var path = UIBezierPath(roundedRect: rect,
                byRoundingCorners: UIRectCorner.AllCorners,
                cornerRadii: CGSize(width: self.radiCantos, height: self.radiCantos))
            path.addClip()
        }
        
        if self.invertir {
            graphPoints=graphPoints.reverse()
        }

        //Dibuixar el degradat del fons
        let context = UIGraphicsGetCurrentContext()
        let colors = [colorInicial.CGColor, colorFinal.CGColor]
        let colorSpace = CGColorSpaceCreateDeviceRGB()
        let colorLocations:[CGFloat] = [0.0, 1.0]

        let gradient = CGGradientCreateWithColors(colorSpace,
            colors,
            colorLocations)
        
        var startPoint = CGPoint.zeroPoint
        var endPoint = CGPoint(x:0, y:self.bounds.height)
        CGContextDrawLinearGradient(context,
            gradient,
            startPoint,
            endPoint,
            0)
        
        if graphPoints.count > 1 {
            //Calcular punts x, o sigui les columnes
            var columnXPoint = { (column:Int) -> CGFloat in
                //Calculate gap between points
                
                let spacer = (width - margeTotal - 4) /
                    CGFloat((self.graphPoints.count - 1))
                var x:CGFloat = CGFloat(column) * spacer
                x += self.margeEsquerra + 2
                return x
            }
            //calcular punts y
            let maxValue = maxElement(graphPoints)
            let minValue = minElement(graphPoints)
            
            //Valor més petit = posició alçada - marge superior
            //valor més elevat = alçada - marge superior - grapthheight
            
            let topGrafic = height - self.margeSuperior
            let bottomGrafic = height - self.margeSuperior - graphHeight
            var columnYPoint = { (graphPoint:Double) -> CGFloat in
                //var y:CGFloat = CGFloat(graphPoint) / CGFloat(maxValue) * graphHeight
                var y:CGFloat = CGFloat(graphPoint-minValue) / CGFloat(maxValue-minValue) * graphHeight
                y = graphHeight + self.margeSuperior - y // Flip the graph
                return y
            }
            
            // draw the line graph
            
            UIColor.whiteColor().setFill()
            UIColor.whiteColor().setStroke()
            
            //set up the points line
            var graphPath = UIBezierPath()
            //go to start of line
            graphPath.moveToPoint(CGPoint(x:columnXPoint(0),
                y:columnYPoint(graphPoints[0])))
            
            //add points for each item in the graphPoints array
            //at the correct (x, y) for the point
            for i in 1..<graphPoints.count {
                let nextPoint = CGPoint(x:columnXPoint(i),
                    y:columnYPoint(graphPoints[i]))
                graphPath.addLineToPoint(nextPoint)
            }
            
            //Create the clipping path for the graph gradient
            
            //1 - save the state of the context (commented out for now)
            CGContextSaveGState(context)
            
            //2 - make a copy of the path
            var clippingPath = graphPath.copy() as! UIBezierPath
            
            //3 - add lines to the copied path to complete the clip area
            clippingPath.addLineToPoint(CGPoint(
                x: columnXPoint(graphPoints.count - 1),
                y:height))
            clippingPath.addLineToPoint(CGPoint(
                x:columnXPoint(0),
                y:height))
            clippingPath.closePath()
            
            //4 - add the clipping path to the context
            clippingPath.addClip()
            
            //5 - check clipping path
            let highestYPoint = columnYPoint(maxValue)
            startPoint = CGPoint(x:margeTotal, y: highestYPoint)
            endPoint = CGPoint(x:margeTotal, y:self.bounds.height)
            
            
            CGContextDrawLinearGradient(context, gradient, startPoint, endPoint, 0)
            
            
            graphPath.lineWidth = 2.0
            graphPath.stroke()
            CGContextRestoreGState(context)
            
            //Draw the circles on top of graph stroke
            for i in 0..<graphPoints.count {
                var point = CGPoint(x:columnXPoint(i), y:columnYPoint(graphPoints[i]))
                point.x -= 5.0/2
                point.y -= 5.0/2
                
                let circle = UIBezierPath(ovalInRect:
                    CGRect(origin: point,
                        size: CGSize(width: 5.0, height: 5.0)))
                circle.fill()
            }
        }
        
        //Draw horizontal graph lines on the top of everything
        var linePath = UIBezierPath()
        
        //top line
        linePath.moveToPoint(CGPoint(x:self.margeEsquerra, y: self.margeSuperior))
        linePath.addLineToPoint(CGPoint(x: width - self.margeDret,
            y:self.margeSuperior))
        
        //center line
        linePath.moveToPoint(CGPoint(x:self.margeEsquerra,
            y: graphHeight/2 + self.margeSuperior))
        linePath.addLineToPoint(CGPoint(x:width - self.margeDret,
            y:graphHeight/2 + self.margeSuperior))
        
        //bottom line
        linePath.moveToPoint(CGPoint(x:self.margeEsquerra,
            y:height - self.margeInferior))
        linePath.addLineToPoint(CGPoint(x:width - self.margeDret,
            y:height - self.margeInferior))
        let color = UIColor(white: 1.0, alpha: 0.3)
        color.setStroke()
        
        linePath.lineWidth = 1.0
        linePath.stroke()
    }
}