//
//  MantenimentController.swift
//  Concot
//
//  Created by Andreu Font Bardolet on 2/6/15.
//  Copyright (c) 2015 Andreu Font Bardolet. All rights reserved.
//

import UIKit
import CoreData
import SwiftForms

class MantenimentController: FormViewController, UINavigationControllerDelegate {
    
    var vehicle: Vehicle! = nil
    var manteniment: Manteniment? = nil
    var registreManteniment: RegistreManteniment? = nil
    var tipusManteniment: String? = nil
    var tipusRow: FormRowType? = nil
    
    struct Static {
        //Selector de tipus
        static let tipusManteniment = "tipusManteniment"
        static let aplicarTipusManteniment = "aplicarTipusManteniment"
        
        //Dades transversals
        static let nomManteniment = "nomManteniment"
        static let tempsDurada = "tempsDurada"
        static let distancia = "distancia"
        
        
        //Dades fer manteniment
        static let dataQueShaFet = "dataQueShaFet"
        static let contaquilometres = "contaquilometres"
        static let cost = "cost"
        
        //Dades consulta
        static let contaquilometresFi = "contaquilometresFi"
        static let dataFi = "dataFi"
    }

    override func viewDidLoad() {
        super.viewDidLoad()
        if manteniment == nil && registreManteniment == nil{
            tipusRow = nil
            self.navigationItem.title = "Afegir manteniment"
            triarTipus()
            self.navigationItem.leftBarButtonItem = UIBarButtonItem(title: "Cancel·lar", style: .Plain, target: self, action: "cancelar:")
        } else  if registreManteniment == nil && manteniment != nil{
            //Carregar manteniment obert
            tipusRow = .Label
            self.navigationItem.title = manteniment!.nom
            mostrarDetallManteniment()
            self.navigationItem.leftBarButtonItem = UIBarButtonItem(title: "Tancar", style: .Plain, target: self, action: "cancelar:")
            //self.navigationItem.rightBarButtonItem = UIBarButtonItem(title: "Desa canvis", style: .Done, target: self, action: "desaCanvis:")
        } else{
            tipusRow = .Label
            //Carregar registre manteniment obert
            self.navigationItem.title = registreManteniment!.nom
            mostrarDetallRegistre()
            //self.navigationItem.rightBarButtonItem = UIBarButtonItem(title: "Desa canvis", style: .Done, target: self, action: "desa:")
            self.navigationItem.leftBarButtonItem = UIBarButtonItem(title: "Tancar", style: .Plain, target: self, action: "cancelar:")
        }
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
    
    required init(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        self.loadForm()
    }
    
    private func mostrarDetallRegistre () {
        //Mostrar/editar les dades d'un registre de manteniment
        tipusManteniment = registreManteniment?.nom
        
        self.form.sections = [seccioInformacioManteniment(), seccioInfoCaducitat(), seccioPropietatsManteniment()]
        tableView.reloadData()
    }
    
    private func mostrarDetallManteniment () {
        //Mostra l'estat d'un manteniment i permet crear un nou registre
        tipusManteniment = manteniment?.nom
        
        let section4 = FormSectionDescriptor()
        var row: FormRowDescriptor! = FormRowDescriptor(tag: Static.aplicarTipusManteniment, rowType: .Button, title: "Fer manteniment")
        row.configuration[FormRowDescriptor.Configuration.DidSelectClosure] = {
                self.ferMantenimentForm()
            } as DidSelectClosure
        section4.addRow(row)
        
        self.form.sections = [seccioEstatDesgast(),  section4, seccioInfoCaducitat(), seccioPropietatsManteniment()]
        tableView.reloadData()
    }
    
    private func ferMantenimentForm() {
        //Des de el detall on es mostra l'estat d'un manteniment, es pot accedir a aquest formulari per crear un registre de manteniment
        self.navigationItem.rightBarButtonItem = UIBarButtonItem(title: "Afegeix", style: .Done, target: self, action: "desaNouRegistre:")
        tipusRow = nil
        self.form.sections = [seccioInformacioManteniment(), seccioInfoCaducitat(), seccioPropietatsManteniment()]
        tableView.reloadData()
    }
    
    private func triat(opcio: String) {
        
        self.navigationItem.rightBarButtonItem = UIBarButtonItem(title: "Afegeix", style: .Done, target: self, action: "desa:")
        
        tipusManteniment = opcio
        
        let seccioCaducitat = seccioInfoCaducitat()
        seccioCaducitat.headerTitle = "Durada de les peces"
        seccioCaducitat.footerTitle = "Indicar quan temps dura una peca fins que caduca i/o els quilòmetres que pot reccorer permet que l'aplicació pugui calcular l'estat del manteniment de \(vehicle.nom) i el cost de desgast de peces al calcular trajectes."
        
        let seccioPropManteniment = seccioPropietatsManteniment()
        seccioPropManteniment.headerTitle = "Informació del manteniment"
        
        let seccioInfoManteniment = seccioInformacioManteniment()
        seccioInfoManteniment.headerTitle = "Registrar últim manteniment fet (recomanat)"
        seccioInfoManteniment.footerTitle = "Al registrar el darrer manteniment, Concot podrà començar a calcular el desgast i costs associats"
        
        self.form.sections = [seccioPropManteniment, seccioCaducitat, seccioInfoManteniment]
        tableView.reloadData()
    }
    
    private func triarTipus(){
        
        //Seleccionar la peca que es vol controlar per mateniment (menys les ja controlades)
        var llistaPeces: [String] = []
        let kLlistaPeces = Manteniments().kLlistaTipus.sorted({ (a, b) -> Bool in
            return b>a
        })
        //Filtrem les peces ja usades
        var apareix = false
        for peca in kLlistaPeces {
            apareix = false
            for manteniment in vehicle.manteniment as! NSMutableSet {
                if (manteniment as! Manteniment).nom == peca {
                    apareix = true
                }
            }
            if apareix == false {
                llistaPeces += [peca]
            }
        }
        let section1 = FormSectionDescriptor()
        
        section1.headerTitle = "Tria sobre què vols fer el mateniment"
        
        var row: FormRowDescriptor! = FormRowDescriptor(tag: Static.tipusManteniment, rowType: .Picker, title: "Manteniment de...")
        row.configuration[FormRowDescriptor.Configuration.Options] = llistaPeces
        row.configuration[FormRowDescriptor.Configuration.TitleFormatterClosure] = { value in
            switch( value ) {
            case "": return ""
            default:
                return value as! String
            }
            } as TitleFormatterClosure
        //row.configuration[FormRowDescriptor.Configuration.DidUpdateClosure] = { value in self.triat(row.value as! String) } as UpdateClosure
        section1.addRow(row)
        
        let row2 = FormRowDescriptor(tag: Static.aplicarTipusManteniment, rowType: .Button, title: "Seleccionar")
        row2.configuration[FormRowDescriptor.Configuration.DidSelectClosure] = {
            
            if row.value != nil {
                self.triat(row.value as! String)
            }
            } as DidSelectClosure
        section1.addRow(row2)
        
        
        self.form.sections = [section1]
        self.tableView.reloadData()
    }
    
    private func loadForm() {
        //Creen el formulari buit, després en el ViewDidLoad es decidirà què s'hi posa
        let form = FormDescriptor()
        self.form = form
    }

    func cancelar(_: UIBarButtonItem!) {
        dismissViewControllerAnimated(true, completion: nil)
    }
    
    func desa(_: UIBarButtonItem!) {
        let dades = form.formValues()
        let valid = self.validarFormulari(dades)
        
        if valid {
            //Comprovem que no existeixi un manteniment amb el mateix nom per el vehicle actual
            let vehicleRequest = NSFetchRequest(entityName: "Manteniment")
            let manteniments = (vehicle.manteniment as! NSMutableSet)
            
            var existeix = false
            for manteniment in manteniments {
                if (manteniment as! Manteniment).nom == tipusManteniment {
                    existeix = true
                }
            }
            
            if existeix != true {
                //Crear nou manteniment
                let appDelegate = (UIApplication.sharedApplication().delegate as! AppDelegate)
                let managedObjectContext = appDelegate.managedObjectContext
                let mantenimentNou = NSEntityDescription.insertNewObjectForEntityForName("Manteniment", inManagedObjectContext: managedObjectContext!) as! Manteniment
                
                registrarNovaPeca(mantenimentNou, dades: dades, context: managedObjectContext!)
                mantenimentNou.vehicle = vehicle
                
                //Comprovar si també cal registrar un manteniment
                if (dades[Static.contaquilometres] as? NSString) != nil || (dades[Static.cost] as? NSString) != nil || (dades[Static.dataQueShaFet] as? NSDate) != nil {
                    registrarNouManteniment(mantenimentNou, dades: dades, context: managedObjectContext!)
                }
                
                
                appDelegate.saveContext()
                dismissViewControllerAnimated(true, completion: nil)
            } else {
                let alert: UIAlertView = UIAlertView(title: "Error", message: "Ja existeix un manteniment amb aquest nom.", delegate: nil, cancelButtonTitle: "Dacrod")
                alert.show()
            }
        }
    }
    
    func desaNouRegistre(_: UIBarButtonItem!) {
        let dades = form.formValues()
        let valid = self.validarFormulari(dades)
        
        if valid {
            if (dades[Static.contaquilometres] as? NSString) != nil || (dades[Static.cost] as? NSString) != nil || (dades[Static.dataQueShaFet] as? NSDate) != nil {
                let appDelegate = (UIApplication.sharedApplication().delegate as! AppDelegate)
                let managedObjectContext = appDelegate.managedObjectContext
                registrarNouManteniment(self.manteniment!, dades: dades, context: managedObjectContext!)
                appDelegate.saveContext()
                self.mostrarDetallManteniment()
                self.navigationItem.rightBarButtonItem = nil
            } else {
                let alert: UIAlertView = UIAlertView(title: "Error", message: "Si es vol registrar el darrer manteniment, concot necessita saber la data que es va fer, els quilòmetres que tenia el vehicle i el cost.", delegate: nil, cancelButtonTitle: "Dacrod")
                alert.show()
            }
        }
    }
    
    private func validarFormulari (dades: NSDictionary) -> Bool {
        var missatge = ""
        
        //if (dades[Static.nomManteniment] as? String ?? "") == "" {
        //    missatge = "Cal indicar un nom que identifiquí aquest manteniment."
        //}
        
        if (dades[Static.contaquilometres] as? NSString) != nil || (dades[Static.cost] as? NSString) != nil || (dades[Static.dataQueShaFet] as? NSDate) != nil {
            if (dades[Static.contaquilometres] as? NSString) == nil || (dades[Static.cost] as? NSString) == nil || (dades[Static.dataQueShaFet] as? NSDate) == nil {
                missatge = "Si es vol registrar el darrer manteniment, concot necessita saber la data que es va fer, els quilòmetres que tenia el vehicle i el cost."
            }
        }
        
        
        if missatge != ""{
            let alert: UIAlertView = UIAlertView(title: "Error", message: missatge, delegate: nil, cancelButtonTitle: "Dacrod")
            alert.show()
            return false
        } else {
            return true
        }
    }
    
    private func registrarNovaPeca(manteniment: Manteniment, dades: NSDictionary, context: NSManagedObjectContext) {
        //Emplenem les dades bàsiques
        manteniment.nom = tipusManteniment! //dades[Static.nomManteniment] as! String
        //manteniment.data = dades[Static.dataQueShaFet] as! NSDate
        //manteniment.distanciaMuntatge = (dades[Static.contaquilometres] as! NSString).toDoubleValue()!
        manteniment.tempsCaducitat = (dades[Static.tempsDurada] as? NSString)?.toDoubleValue()
        manteniment.distanciaMaxima = (dades[Static.distancia] as? NSString)?.toDoubleValue()
        //manteniment.cost = (dades[Static.cost] as! NSString).toDoubleValue()!
        //manteniment.moneda = Moneda(string: vehicle.moneda).simbol
        
        //Emplenem les propietats dinàmiques de cada tipus de manteniment
        let tipus = Manteniments(tipus: tipusManteniment!)
        
        for var i = 0; i < tipus.nomsValor.count; ++i {
            let propietatNova = NSEntityDescription.insertNewObjectForEntityForName("PropietatsManteniment", inManagedObjectContext: context) as! PropietatsManteniment
            
            propietatNova.nom = tipus.nomsValor[i]
            propietatNova.tipus = tipus.tipusValor[i]
            switch propietatNova.tipus {
            case "String" :
                propietatNova.valor = dades[tipus.nomsValor[i]] as? String
            case "Double" :
                propietatNova.valor = (dades[tipus.nomsValor[i]] as? NSString)?.toDoubleValue()
            case "Text" :
                propietatNova.valor = dades[tipus.nomsValor[i]] as? String
            case "Bool" :
                propietatNova.valor = dades[tipus.nomsValor[i]] as? Bool
            case "Number" :
                propietatNova.valor = (dades[tipus.nomsValor[i]] as? NSString)?.toDoubleValue()
            default: break
            }
            //propietatNova.valor = dades[tipus.nomsValor[i]]// as? NSData
            
            propietatNova.manteniment = manteniment
        }
        
    }
    
    private func registrarNouManteniment(manteniment: Manteniment, dades: NSDictionary, context: NSManagedObjectContext) {
        
        let darrer = manteniment.obtenirUltimManteniment()
        
        let registreManteniment = NSEntityDescription.insertNewObjectForEntityForName("RegistreManteniment", inManagedObjectContext: context) as! RegistreManteniment
        
        
        registreManteniment.nom = manteniment.nom
        registreManteniment.distanciaMaxima = manteniment.distanciaMaxima
        registreManteniment.distanciaMuntatgeInicial = (dades[Static.contaquilometres] as! NSString).toDoubleValue()!
        registreManteniment.data = dades[Static.dataQueShaFet] as! NSDate
        registreManteniment.cost = (dades[Static.cost] as! NSString).toDoubleValue()!
        registreManteniment.moneda = Moneda(string: vehicle.moneda).simbol
        registreManteniment.distanciaMuntatgeFinal = nil
        registreManteniment.dataCanvi = nil
        registreManteniment.tempsCaducitat = manteniment.tempsCaducitat
        registreManteniment.vehicle = manteniment.vehicle
        
        for propietat in (manteniment.propietats as! NSMutableSet) {
            let propietatNova = NSEntityDescription.insertNewObjectForEntityForName("PropietatsRegistreManteniment", inManagedObjectContext: context) as! PropietatsRegistreManteniment
            
            propietatNova.nom = (propietat as! PropietatsManteniment).nom
            propietatNova.tipus = (propietat as! PropietatsManteniment).tipus
            propietatNova.valor = (propietat as! PropietatsManteniment).valor
            
            propietatNova.manteniment = registreManteniment
        }
        
        //Hi ha manteniments anteriors registrats?
        if darrer != nil {
            //Actualitzar les dades
            darrer!.distanciaMuntatgeFinal = registreManteniment.distanciaMuntatgeInicial
            darrer!.dataCanvi = registreManteniment.data
        }
        

    }
    
    //MARK - Seccions de dades
    private func seccioEstatDesgast() -> FormSectionDescriptor {
        let section = FormSectionDescriptor()
        section.headerTitle = "Estat"
        if let darrer = manteniment?.obtenirUltimManteniment() {
            var row: FormRowDescriptor! = FormRowDescriptor(tag: "", rowType: .Label, title: "Nivell de desgast")
            row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
            if let desgast = manteniment?.desgast{
                row.value = (desgast * 100.0).format(".2") + "%"
            } else {
                row.value = "Desconegut"
            }
            section.addRow(row)
            
            row = FormRowDescriptor(tag: "", rowType: .Label, title: "\(Distancia(string: vehicle.unitatKm).plural) al muntar")
            row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
            row.value = Double(darrer.distanciaMuntatgeInicial).format(".2") + " \(Distancia(string: vehicle.unitatKm).unitatMesura)"
            section.addRow(row)
            
            row = FormRowDescriptor(tag: "", rowType: .Label, title: "Proper canvi")
            row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
            row.value = (manteniment?.properCanvi?.format(".2") ?? "--") + " \(Distancia(string: vehicle.unitatKm).unitatMesura)"
            section.addRow(row)
            
            row = FormRowDescriptor(tag: "", rowType: .Label, title: "\(Distancia(string: vehicle.unitatKm).plural) restants")
            row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
            row.value = (manteniment?.distanciaRestant?.format(".2") ?? "--") + " \(Distancia(string: vehicle.unitatKm).unitatMesura)"
            section.addRow(row)
            
            row = FormRowDescriptor(tag: "", rowType: .Label, title: "Data de muntatge")
            row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
            row.value = (manteniment?.dataActual)?.toString() ?? "Desgonegut"
            section.addRow(row)
            
            row = FormRowDescriptor(tag: "", rowType: .Label, title: "Data proper canvi")
            row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
            row.value = (manteniment?.dataProperCanvi)?.toString() ?? "Desconegut"
            section.addRow(row)
            
            row = FormRowDescriptor(tag: "", rowType: .Label, title: "Mesos restants")
            row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
            if let tempsRestant = manteniment?.tempsRestant {
                let doubleRestant = Double(tempsRestant)
                row.value = doubleRestant.format(".0") + " Mesos"
            } else {
                row.value = "Desconegut"
            }
            
            section.addRow(row)
        } else {
            section.footerTitle = "No hi ha cap registre d'aquest manteniment per saber-ne l'estat."
        }
        
        return section
    }
    
    private func seccioInfoCaducitat() -> FormSectionDescriptor {
        let section = FormSectionDescriptor()
        section.headerTitle = "Informació de la caducitat"
        
        var tempsDuradaValor: String? = nil
        var distanciaValor: String? = nil
        
        //Dades a mostrar segons estat del formulari
        if registreManteniment != nil {
            //S'està consultant o editant un manteniment
            tempsDuradaValor = (registreManteniment!.tempsCaducitat as? Double)?.format(".0")
            distanciaValor = (registreManteniment!.distanciaMaxima as? Double)?.format(".2")
        } else if manteniment != nil {
            //S'està registrant un manteniment nou per una peça ja existent
            tempsDuradaValor = (manteniment!.tempsCaducitat as? Double)?.format(".0")
            distanciaValor = (manteniment!.distanciaMaxima as? Double)?.format(".2")
            
        }
        
        var tempsDuradaPlaceholer = tempsDuradaValor ?? "12"
        var distanciaPlaceholder = distanciaValor ?? "15000"
        
        var row: FormRowDescriptor! = FormRowDescriptor(tag: Static.tempsDurada, rowType: tipusRow ?? .Number, title: "Caducitat (en mesos)", placeholder: tempsDuradaPlaceholer)
        row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
        row.value = tempsDuradaValor
        section.addRow(row)
        
        row = FormRowDescriptor(tag: Static.distancia, rowType: tipusRow ?? .Decimal, title: "Distància", placeholder: distanciaPlaceholder)
        row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
        row.value = distanciaValor
        section.addRow(row)
        
        if registreManteniment != nil && registreManteniment?.distanciaMuntatgeFinal != nil{
            //S'està consultant un registre de manteniment manteniment
            row = FormRowDescriptor(tag: Static.distancia, rowType: tipusRow ?? .Date, title: "Ultim dia usat", placeholder: distanciaPlaceholder)
            row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
            row.value = registreManteniment?.dataCanvi?.toString()
            section.addRow(row)
            
            row = FormRowDescriptor(tag: Static.distancia, rowType: tipusRow ?? .Decimal, title: "Distància final", placeholder: distanciaPlaceholder)
            row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
            row.value = Double(registreManteniment!.distanciaMuntatgeFinal!).format(".2")
            section.addRow(row)
        }
        
        return section
    }
    
    private func seccioInformacioManteniment() -> FormSectionDescriptor {
        let section = FormSectionDescriptor()
        
        var data: NSDate? = nil
        var distanciaMuntatgeInicial: String? = nil
        var distanciaMuntatgeInicialPlaceholder: String = (vehicle.obtenirKm() as? Double)?.format(".2") ?? "15000"
        var cost: String? = nil
        var costPlaceholder: String = "120,34"
        
        //Dades a mostrar segons estat del formulari
        if registreManteniment != nil {
            //S'està consultant o editant un manteniment
            data = registreManteniment!.data
            distanciaMuntatgeInicial = (registreManteniment!.distanciaMuntatgeInicial as! Double).format(".2")
            distanciaMuntatgeInicialPlaceholder = distanciaMuntatgeInicial ?? "15000"
            cost = (registreManteniment!.cost as! Double).format(".2")
            costPlaceholder = cost ?? "15000"
        } else if manteniment != nil {
            //S'està registrant un manteniment nou per una peça ja existent
            data = NSDate()
            distanciaMuntatgeInicial = (vehicle.obtenirKm() as? Double)?.format(".2")
        } else {
            //S'està creant un nou manteniment i opcionalment es pot registrar el darrer manteniment
            //No cal fer res, la preinicialització de les dades ja és suficient
        }
        
        section.headerTitle = "Informació del manteniment"
        var row: FormRowDescriptor! = FormRowDescriptor(tag: Static.dataQueShaFet, rowType: tipusRow ?? .Date, title: "Data")
        if tipusRow != nil {
            row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
            row.value = data?.toString()
        } else {
            row.value = data
        }
        section.addRow(row)
        
        row = FormRowDescriptor(tag: Static.contaquilometres, rowType: tipusRow ?? .Decimal, title: "Contaquilòmetres")
        if tipusRow == nil {
            row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.placeholder" : "\(distanciaMuntatgeInicialPlaceholder)", "textField.textAlignment" : NSTextAlignment.Right.rawValue]
        } else {
            row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
        }
        row.value = distanciaMuntatgeInicial
        section.addRow(row)
        
        row = FormRowDescriptor(tag: Static.cost, rowType: tipusRow ?? .Decimal, title: "Cost")
        if tipusRow == nil {
            row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.placeholder" : "\(costPlaceholder)", "textField.textAlignment" : NSTextAlignment.Right.rawValue]
        } else {
            row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
        }
        row.value = cost
        section.addRow(row)
        
        return section
    }
    
    private func seccioPropietatsManteniment() -> FormSectionDescriptor {
        let seccio = FormSectionDescriptor()
        
        seccio.headerTitle = "Detalls"
        
        var propietats = NSMutableSet()
        
        if registreManteniment != nil {
            propietats = registreManteniment?.propietats as! NSMutableSet
        } else if manteniment != nil {
            propietats = manteniment?.propietats as! NSMutableSet
        } else {
            let tipusPropietats = Manteniments(tipus: tipusManteniment!)
            var diccionari = Dictionary<String, String>()
            for var i = 0; i < tipusPropietats.nomsValor.count; ++i {
                diccionari = ["nom":tipusPropietats.nomsValor[i],"tipus":tipusPropietats.tipusValor[i],"valor":tipusPropietats.placeholder[i]]
                propietats.addObject(diccionari)
            }
        }
        
        for propietat in propietats {
            var tipus = ""
            var valor: AnyObject? = nil
            var nom = ""
            if registreManteniment != nil {
                let valors = propietat as! PropietatsRegistreManteniment
                nom = valors.nom
                valor = valors.valor
                tipus = valors.tipus
            } else if manteniment != nil {
                let valors = propietat as! PropietatsManteniment
                nom = valors.nom
                valor = valors.valor
                tipus = valors.tipus
            } else {
                let valors = propietat as! Dictionary<String, String>
                nom = valors["nom"]!
                tipus = valors["tipus"]!
                if tipus == "Double" || tipus == "Number" {
                    valor = NSString(string: valors["valor"]!).toDoubleValue()
                    
                } else {
                    valor = valors["valor"]
                }
            }
            
            switch tipus {
                case "String":
                    let row: FormRowDescriptor! = FormRowDescriptor(tag: nom, rowType: tipusRow ?? .Name, title: nom, placeholder: valor as? String ?? "")
                    row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
                    row.value = propietat.valor as? String
                    seccio.addRow(row)
                case "Text":
                    let row: FormRowDescriptor! = FormRowDescriptor(tag: nom, rowType: tipusRow ?? .MultilineText, title: nom)
                    row.value = propietat.valor as? String
                    seccio.addRow(row)
                case "Bool":
                    let row: FormRowDescriptor! = FormRowDescriptor(tag: nom, rowType: .BooleanSwitch, title: nom)
                    if tipusRow != nil {
                        row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["switchView.enabled" : false]
                    }
                    row.value = propietat.valor as? Bool
                    seccio.addRow(row)
                case "Double":
                    let placeholder = (valor as? Double)?.format(".2") ?? ""
                    let row: FormRowDescriptor! = FormRowDescriptor(tag: nom, rowType: tipusRow ?? .Decimal, title: nom, placeholder: placeholder)
                    row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
                    row.value = (propietat.valor as? Double)?.format(".2")
                    seccio.addRow(row)
                case "Number":
                    let placeholder = (valor as? Double)?.format(".2") ?? ""
                    let row: FormRowDescriptor! = FormRowDescriptor(tag: nom, rowType: tipusRow ?? .Number, title: nom, placeholder: placeholder)
                    row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
                    row.value = (propietat.valor as? Double)?.format(".2")
                    seccio.addRow(row)
                default: break
            }
        }
        return seccio
    }
}
