//
//  OmplirDipositController.swift
//  Concot
//
//  Created by Andreu Font Bardolet on 10/5/15.
//  Copyright (c) 2015 Andreu Font Bardolet. All rights reserved.
//

import UIKit
import CoreData

class OmplirDipositController: UITableViewController, UINavigationControllerDelegate {

    @IBOutlet weak var dataInputTextField: UITextField!
    @IBOutlet weak var kmInputTextField: UITextField!
    @IBOutlet weak var litresInputTextField: UITextField!
    @IBOutlet weak var costInputTextField: UITextField!
    @IBOutlet weak var preuInputTextField: UITextField!
    @IBOutlet weak var dipositSegmentedControl: UISegmentedControl!
    @IBOutlet weak var carburantSegmentedControl: UISegmentedControl!
    
    
    var autoLitres: Bool = false
    var autoCost: Bool = false
    var autoPreu: Bool = false
    
    var barraMenu: UIToolbar = UIToolbar(frame: CGRectMake(0, 0, 320, 50))
    
    var vehicle: Vehicle!
    var carburants: [Carburant] = []
    
    let formatter = NSDateFormatter()
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        //Preparem la barra de menú del teclat amb el botó OK
        barraMenu.barStyle = UIBarStyle.Default
        var flexSpace = UIBarButtonItem(barButtonSystemItem: UIBarButtonSystemItem.FlexibleSpace, target: nil, action: nil)
        var done: UIBarButtonItem = UIBarButtonItem(title: "Ok", style: UIBarButtonItemStyle.Done, target: self, action: Selector("okApretat"))
        var items = NSMutableArray()
        items.addObject(flexSpace)
        items.addObject(done)
        barraMenu.items = items as [AnyObject]
        barraMenu.sizeToFit()
        
        //Afegim aquest menú a tots els textField
        dataInputTextField.inputAccessoryView = barraMenu
        kmInputTextField.inputAccessoryView = barraMenu
        litresInputTextField.inputAccessoryView = barraMenu
        costInputTextField.inputAccessoryView = barraMenu
        preuInputTextField.inputAccessoryView = barraMenu
        
        //Activem que es calculin automàticament els diferents valors
        autoCost = false
        autoLitres = false
        autoPreu = false
        
        let date = NSDate()
        formatter.locale = NSLocale(localeIdentifier: "ES_es")
        formatter.dateFormat = "dd/MM/yyyy hh:mm:ss"
        dataInputTextField.text = formatter.stringFromDate(date)
        
        kmInputTextField.placeholder = "\(vehicle.obtenirKm())"
        
        //Obtenir darrer preu de carburant per proposar-lo com a preu
        let darrerEmplenament = vehicle.obtenirDarrerEmplenament()
        preuInputTextField.text = darrerEmplenament?.preuDouble.format(".3")
        
        // GESTIÓ LLISTA CARBURANTS
        //obtenir carburants del vehicle
        carburants = vehicle.obtenirCarburants()
        let darrerCarburant = darrerEmplenament?.carburant
        
        //Emplenar el segmented control amb aquests
        carburantSegmentedControl.removeAllSegments()
        var index = 0
        for carburant in carburants {
            carburantSegmentedControl.insertSegmentWithTitle(carburant.nom, atIndex: index, animated: true)
            //Marcar darrer carburant usat si es pot
            if darrerCarburant?.nom == carburant.nom {
                carburantSegmentedControl.selectedSegmentIndex = index
            } else if index == 0 {
                carburantSegmentedControl.selectedSegmentIndex = 0
            }
            index += 1
        }

        // Uncomment the following line to preserve selection between presentations
        // self.clearsSelectionOnViewWillAppear = false

        // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
        // self.navigationItem.rightBarButtonItem = self.editButtonItem()
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    override func viewDidAppear(animated: Bool) {
        kmInputTextField.becomeFirstResponder()
    }
    
    @IBAction func cancelarBarButtonPressed(sender: UIBarButtonItem) {
        self.dismissViewControllerAnimated(true, completion: nil)
    }
    
    
    @IBAction func desaBarButtonPressed(sender: UIBarButtonItem) {
        
        //Comprovar que tots els camps estàn correctes
        let data = dataInputTextField.text
        let km = (kmInputTextField.text as NSString).toDoubleValue()!
        let litres = (litresInputTextField.text as NSString).toDoubleValue()!
        let cost = (costInputTextField.text as NSString).toDoubleValue()!
        let preu = (preuInputTextField.text as NSString).toDoubleValue()!
        let darrerEmplenament = vehicle.obtenirDarrerEmplenament()
        
        if data != "" && km != 0 && litres != 0 && cost != 0 && preu != 0 {
            
            //obtenir true/false del diposit ple
            var ple: Bool = true
            if dipositSegmentedControl.selectedSegmentIndex == 1 {
                ple = false
            }
            
            
            //Crear nou emplenament
            let appDelegate = (UIApplication.sharedApplication().delegate as! AppDelegate)
            let managedObjectContext = appDelegate.managedObjectContext
            
            let emplenament = NSEntityDescription.insertNewObjectForEntityForName("Emplenament", inManagedObjectContext: managedObjectContext!) as! Emplenament
            
            
            //Omplir dades d'emplenament
            emplenament.guardarDataEmplenament(data)
            emplenament.km = km
            emplenament.litres = litres
            emplenament.cost = cost
            emplenament.preu = preu
            emplenament.ple = NSNumber(bool: ple)
            emplenament.carburant = carburants[carburantSegmentedControl.selectedSegmentIndex]
            emplenament.vehicle = vehicle
            emplenament.moneda = vehicle.moneda
            
            emplenament.anterior = darrerEmplenament
            
            
            //guardar i Tancar la pantalla
            appDelegate.saveContext()
            self.dismissViewControllerAnimated(true, completion: nil)
        }
    }
    
    @IBAction func entrarDataTextField(sender: UITextField) {
        var datePickerView:UIDatePicker = UIDatePicker()
        datePickerView.datePickerMode = UIDatePickerMode.DateAndTime
        sender.inputView = datePickerView
        datePickerView.addTarget(self, action: Selector("datePickerValueChanged:"), forControlEvents: UIControlEvents.ValueChanged)
        //sender.inputAccessoryView = barraMenu
    }
    
    func okApretat(){
        dataInputTextField.resignFirstResponder()
        kmInputTextField.resignFirstResponder()
        litresInputTextField.resignFirstResponder()
        costInputTextField.resignFirstResponder()
        preuInputTextField.resignFirstResponder()
    }
    
    func datePickerValueChanged(sender: UIDatePicker) {
        dataInputTextField.text = formatter.stringFromDate(sender.date)
    }
    
    
    // MARK: - Control de canvis i autocàlcul de valors
    @IBAction func litresCanvien() {
        if let litres: Double = (litresInputTextField.text as NSString).toDoubleValue() {
            let cost: Double = (costInputTextField.text as NSString).toDoubleValue()!
            let preu: Double = (preuInputTextField.text as NSString).toDoubleValue()!
            
            autoLitres = false
            
            if (cost == 0 || autoCost ) {
                let calcul: Double = litres * preu
                costInputTextField.text = calcul.format(".2")
                autoCost = true
            } else if (preu == 0 || autoPreu ) {
                let calcul: Double = cost / litres
                preuInputTextField.text = calcul.format(".3")
                autoPreu = true
            } else {
                let calcul: Double = litres * preu
                costInputTextField.text = calcul.format(".2")
                autoCost = true
            }
        }
    }
    
    @IBAction func costCanvia() {
        if let cost: Double = (costInputTextField.text as NSString).toDoubleValue() {
            let litres: Double = (litresInputTextField.text as NSString).toDoubleValue()!
            let preu: Double = (preuInputTextField.text as NSString).toDoubleValue()!
            
            autoCost = false
            
            if (preu == 0 || autoPreu ) {
                let calcul: Double = cost / litres
                preuInputTextField.text = calcul.format(".3")
                autoPreu = true
            } else if (litres == 0 || autoLitres ) {
                let calcul: Double = cost / preu
                litresInputTextField.text = calcul.format(".2")
                autoLitres = true
            } else {
                let calcul: Double = cost / litres
                preuInputTextField.text = calcul.format(".3")
                autoPreu = true
            }
        }
    }
    
    @IBAction func preuCanvia() {
        if let preu: Double = (preuInputTextField.text as NSString).toDoubleValue() {
            if (costInputTextField.text != "" || litresInputTextField.text != ""){
                let cost: Double = (costInputTextField.text as NSString).toDoubleValue()!
                let litres: Double = (litresInputTextField.text as NSString).toDoubleValue()!
                
                autoPreu = false
                if (cost == 0 || autoCost ) {
                    let calcul: Double = litres * preu
                    costInputTextField.text = calcul.format(".2")
                    autoCost = true
                } else if (litres == 0 || autoLitres ) {
                    let calcul: Double = cost / preu
                    litresInputTextField.text = calcul.format(".2")
                    autoLitres = true
                } else {
                    let calcul: Double = litres * preu
                    costInputTextField.text = calcul.format(".2")
                    autoCost = true
                }
            }
        }
    }
    
    
    // MARK: - Autoselecció de text dels textfield
    
    @IBAction func seleccionarTot(sender: UITextField) {
        sender.selectedTextRange = sender.textRangeFromPosition(sender.beginningOfDocument, toPosition: sender.endOfDocument)
    }
    
    

    

}
