//
//  Utilitats.swift
//  Concot
//
//  Created by Andreu Font Bardolet on 10/5/15.
//  Copyright (c) 2015 Andreu Font Bardolet. All rights reserved.
//


/*
En aquest document, hi ha extensions de les funcións dels tipus de dades propis de Swift,
com Double, NSString i NSDate, que ajudaràn a fer càlculs i transformacións de tipus de dades
*/

import Foundation

extension Double {
    func format(f: String) -> String {
        //Funció que amplia el tipus de dada Double, permetent passar un número a string
        //indicant el nombre de decimals que es vol
        
        //Generem un string a partir del valor numèric, amb el número de decimals indicat
        let string = NSString(format: "%\(f)f", self) as String
        
        //Editem l'strinc substituint (si hi és) el punt decimal (americà) per la coma decimal (europea)
        let stringComa = string.stringByReplacingOccurrencesOfString(".", withString: ",")
        
        //Per netejar zeros inútils darrera la coma, obtenim el númer de decimals que s'han demanat
        let decimals = f.stringByReplacingOccurrencesOfString(".", withString: "").toInt()
        
        //Segons el número de decimals demanats, generem un string amb una coma i zeros a la dreta (exemple: ,000)
        var stringEliminar = ""
        for var i = 0; i<decimals; ++i {
            stringEliminar += "0"
        }
        
        //Si ni ha zeros, agefim la coma
        if stringEliminar != "" {
            stringEliminar = "," + stringEliminar
        }
        
        //Busquem a l'string a tetornar si els decimals són tot zeros com en l'exemple i els eliminem
        return stringComa.stringByReplacingOccurrencesOfString(stringEliminar, withString: "")
    }
}

extension NSString {
    func toDoubleValue() -> Double? {
        //Funció que ambplia el tipus de dades NSString i permet transforma un string a double
        
        //Canviem la coma decima (europea) per el punt decimal (america)
        let newString = self.stringByReplacingOccurrencesOfString(",", withString: ".")
        
        //Retornem el valor en double
        return (newString as NSString).doubleValue
    }
}

extension NSDate {
    
    //Les següents funcions serveixen per obtenir el número de dies, mesos, setmanes, etc. entre dues dades, retornen un integer com a resultat
    func yearsFrom(date:NSDate) -> Int{
        return NSCalendar.currentCalendar().components(NSCalendarUnit.CalendarUnitYear, fromDate: date, toDate: self, options: nil).year
    }
    func monthsFrom(date:NSDate) -> Int{
        return NSCalendar.currentCalendar().components(NSCalendarUnit.CalendarUnitMonth, fromDate: date, toDate: self, options: nil).month
    }
    func weeksFrom(date:NSDate) -> Int{
        return NSCalendar.currentCalendar().components(NSCalendarUnit.CalendarUnitWeekOfYear, fromDate: date, toDate: self, options: nil).weekOfYear
    }
    func daysFrom(date:NSDate) -> Int{
        return NSCalendar.currentCalendar().components(NSCalendarUnit.CalendarUnitDay, fromDate: date, toDate: self, options: nil).day
    }
    func hoursFrom(date:NSDate) -> Int{
        return NSCalendar.currentCalendar().components(NSCalendarUnit.CalendarUnitHour, fromDate: date, toDate: self, options: nil).hour
    }
    func minutesFrom(date:NSDate) -> Int{
        return NSCalendar.currentCalendar().components(NSCalendarUnit.CalendarUnitMinute, fromDate: date, toDate: self, options: nil).minute
    }
    func secondsFrom(date:NSDate) -> Int{
        return NSCalendar.currentCalendar().components(NSCalendarUnit.CalendarUnitSecond, fromDate: date, toDate: self, options: nil).second
    }
    
    func toString() -> String {
        //Aquesta funció passa una data a format string, usant el calendari espanyol i el fomrat dd/MM/yyy
        let formatter = NSDateFormatter()
        formatter.locale = NSLocale(localeIdentifier: "ES_es")
        //formatter.dateFormat = "E, dd MMM yyyy HH:mm:ss Z"
        formatter.dateFormat = "dd/MM/yyyy"
        return formatter.stringFromDate(self)
    }
}