//
//  VehicleController.swift
//  Concot
//
//  Created by Andreu Font Bardolet on 28/5/15.
//  Copyright (c) 2015 Andreu Font Bardolet. All rights reserved.
//

import UIKit
import CoreData
import MobileCoreServices
import SwiftForms

class VehicleController: FormViewController, UIImagePickerControllerDelegate, UINavigationControllerDelegate {
    
    var vehicle: Vehicle! = nil
    
    var botoDret: UIBarButtonItem!
    var botoEsquerra: UIBarButtonItem!
    
    struct Static {
        //Bàsic requirit
        static let foto = "foto" //Nom de vehicle + foto(opcional)
        
        //Tipus d'unitats i carburants a usar (valors predecidits, canvi opcional)
        static let moneda = "moneda"
        static let distancia = "distancia"
        static let carburant = "carburant"
        
        //Data i preu d'adquisició del vehicle (emplenament opcional
        static let compra = "compra"
        static let adquisicio = "adquisicio"
        static let preu = "preu"
        
        //Informació assegurança (emplenament opcional)
        static let asseguradora = "asseguradora"
        static let asseguradoraTipusPagament = "asseguradoraTipusPagament"
        static let asseguradoraPreu = "asseguradoraPreu"
        static let asseguradoraPolissa = "asseguradoraPolissa"
        static let asseguradoraFranquicia = "asseguradoraFranquicia"
        static let asseguradoraTipus = "asseguradoraTipus"
        static let asseguradoraNotes = "asseguradoraNotes"
        static let asseguradoraMesPagament = "asseguradoraMesPagament"
        
        //Informació impost municipal (emplenament opcional)
        static let impostPagament = "impostPagament"
        static let impostPreu = "impostPreu"
        static let impostMesPagament = "impostMesPagament"
        
        //Altres dades
        static let marca = "marca"
        static let model = "model"
        static let matricula = "matricula"
        static let portes = "portes"
        
    }
    
    required init(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        self.loadForm()
    }

    override func viewDidLoad() {
        super.viewDidLoad()
        
        if let tabBar = self.tabBarController as? TabBarController {
            vehicle = tabBar.vehicle
            self.navigationItem.title = "Informació"
            
            //Emplenem el formulari amb l'informació actual del vehicle
            self.emplenarForm()
            
            self.navigationItem.rightBarButtonItem = UIBarButtonItem(title: "Desa canvis", style: .Done, target: self, action: "desa_canvis:")
            self.navigationItem.leftBarButtonItem = UIBarButtonItem(title: "Desfer canvis", style: .Plain, target: self, action: "desfer_canvis:")
            
        } else {
            //Valors per defecte
            self.navigationItem.title = "Afegir Vehicle"
            form.sections[1].rows[0].value = "€"
            form.sections[1].rows[1].value = "km"
            var carburants: NSMutableArray = []
            carburants.addObject("Diesel")
            carburants.addObject("Diesel Plus")
            form.sections[1].rows[2].value = carburants as NSArray
            
            self.navigationItem.rightBarButtonItem = UIBarButtonItem(title: "Desa", style: .Done, target: self, action: "desa:")
            self.navigationItem.leftBarButtonItem = UIBarButtonItem(title: "Cancel·lar", style: .Plain, target: self, action: "cancelar:")
        }
    }
    
    override func viewDidDisappear(animated: Bool) {
        //Desfer canvis no guardats
        emplenarForm()
        self.tableView.reloadData()
    }

    private func loadForm() {
        
        let form = FormDescriptor()
        
        //if let vehicle = vehicleObert {
        
        //form.title = "Example Form"
        
        //Secció 1, introducció de foto i nom de vehicle
        let section1 = FormSectionDescriptor()
        
        var row: FormRowDescriptor! = FormRowDescriptor(tag: Static.foto, rowType: .Foto, title: "")
        row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.font" : UIFont.boldSystemFontOfSize(25.0), "textField.placeholder" : "Nom del vehicle", "textField.textAlignment" : NSTextAlignment.Left.rawValue]
        row.configuration[FormRowDescriptor.Configuration.Required] = true
        //row.configuration[FormRowDescriptor.Configuration.Image] = foto
        section1.addRow(row)
        
        //Secció 2, selecció d'unitats del vehicle (Moneda, distància) i tipus de carburant a usar
        let section2 = FormSectionDescriptor()
        
        row = FormRowDescriptor(tag: Static.moneda, rowType: .Picker, title: "Moneda")
        row.configuration[FormRowDescriptor.Configuration.Options] = ["€", "$"]
        row.configuration[FormRowDescriptor.Configuration.TitleFormatterClosure] = { value in
            switch( value ) {
            case "€":
                return "Euro"
            case "$":
                return "Dollar"
            default:
                return nil
            }
            } as TitleFormatterClosure
        
        row.value = "€"
        section2.addRow(row)
        
        row = FormRowDescriptor(tag: Static.distancia, rowType: .Picker, title: "Distància")
        row.configuration[FormRowDescriptor.Configuration.Options] = ["km", "mi"]
        row.configuration[FormRowDescriptor.Configuration.TitleFormatterClosure] = { value in
            switch( value ) {
            case "km":
                return "Quilòmetres"
            case "mi":
                return "Milles"
            default:
                return nil
            }
            } as TitleFormatterClosure
        
        row.value = "km"
        section2.addRow(row)
        
        row = FormRowDescriptor(tag: Static.carburant, rowType: .MultipleSelector, title: "Caburants")
        row.configuration[FormRowDescriptor.Configuration.Options] = Carburants().kCarburants
        row.configuration[FormRowDescriptor.Configuration.AllowsMultipleSelection] = true
        row.configuration[FormRowDescriptor.Configuration.TitleFormatterClosure] = { value in
            switch( value ) {
            default:
                return value as! String
            }
            } as TitleFormatterClosure
        //row.value = (["Diesel", "Diesel Plus"]) as NSMutableArray
        
        section2.headerTitle = "Unitats de mesura i carburants a usar"
        
        section2.addRow(row)
        
        //Data de compra i Preu
        let section3 = FormSectionDescriptor()
        
        row = FormRowDescriptor(tag: Static.compra, rowType: .Date, title: "Data de compra")
        section3.addRow(row)
        
        row = FormRowDescriptor(tag: Static.adquisicio, rowType: .Date, title: "Data d'adquisició")
        section3.addRow(row)
        
        row = FormRowDescriptor(tag: Static.preu, rowType: .Decimal, title: "Preu")
        row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.placeholder" : "e.x. 10356,25", "textField.textAlignment" : NSTextAlignment.Right.rawValue]
        section3.addRow(row)
        
        section3.headerTitle = "Dades d'adquisició del vehicle (Opcional)"
        
        
        //Dades de l'assegurança del vehicle
        let section4 = FormSectionDescriptor()
        row = FormRowDescriptor(tag: Static.asseguradora, rowType: .Name, title: "Asseguradora")
        row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.placeholder" : "e.x. Mapfre", "textField.textAlignment" : NSTextAlignment.Right.rawValue]
        section4.addRow(row)
        
        row = FormRowDescriptor(tag: Static.asseguradoraTipusPagament, rowType: .SegmentedControl, title: "Pagament")
        row.configuration[FormRowDescriptor.Configuration.Options] = ["Anual", "Bimestral", "Trimestral"]
        row.configuration[FormRowDescriptor.Configuration.TitleFormatterClosure] = { value in
            switch( value ) {
            case "Anual":
                return "Anual"
            case "Bimestral":
                return "Bimestral"
            case "Trimestral":
                return "Trimestral"
            default:
                return nil
            }
        } as TitleFormatterClosure
        section4.addRow(row)
        
        row = FormRowDescriptor(tag: Static.asseguradoraMesPagament, rowType: .Picker, title: "Primer mes de pagament")
        row.configuration[FormRowDescriptor.Configuration.Options] = ["Gener", "Febrer", "Març", "Abril", "Maig", "Juny", "Juliol", "Agost", "Setembre", "Octubre", "Novembre", "Desembre"]
        row.configuration[FormRowDescriptor.Configuration.TitleFormatterClosure] = { value in
            switch( value ) {
            case "Gener":
                return "Gener"
            case "Febrer":
                return "Febrer"
            case "Març":
                return "Març"
            case "Abril":
                return "Abril"
            case "Maig":
                return "Maig"
            case "Juny":
                return "Juny"
            case "Juliol":
                return "Juliol"
            case "Agost":
                return "Agost"
            case "Setembre":
                return "Setembre"
            case "Octubre":
                return "Octubre"
            case "Novembre":
                return "Novembre"
            case "Desembre":
                return "Desembre"
            default:
                return nil
            }
            } as TitleFormatterClosure
        row.value = ""
        section4.addRow(row)
        
        row = FormRowDescriptor(tag: Static.asseguradoraPreu, rowType: .Decimal, title: "Preu")
        row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.placeholder" : "e.x. 235,04", "textField.textAlignment" : NSTextAlignment.Right.rawValue]
        section4.addRow(row)
        
        row = FormRowDescriptor(tag: Static.asseguradoraFranquicia, rowType: .Decimal, title: "Franquícia")
        row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.placeholder" : "e.x. 1400", "textField.textAlignment" : NSTextAlignment.Right.rawValue]
        section4.addRow(row)
        
        row = FormRowDescriptor(tag: Static.asseguradoraTipus, rowType: .SegmentedControl, title: "Tipus")
        row.configuration[FormRowDescriptor.Configuration.Options] = ["Tercers", "Tot Risc"]
        row.configuration[FormRowDescriptor.Configuration.TitleFormatterClosure] = { value in
            switch( value ) {
            case "Tercers":
                return "Tercers"
            case "Tot Risc":
                return "Tot Risc"
            default:
                return nil
            }
            } as TitleFormatterClosure
        section4.addRow(row)
        
        row = FormRowDescriptor(tag: Static.asseguradoraPolissa, rowType: .Number, title: "Nº de pòlissa")
        row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.placeholder" : "e.x. 2948503", "textField.textAlignment" : NSTextAlignment.Right.rawValue]
        section4.addRow(row)
        
        row = FormRowDescriptor(tag: Static.asseguradoraNotes, rowType: .MultilineText, title: "Notes")
        section4.headerTitle = "Multiline TextView"
        section4.addRow(row)
        
        section4.headerTitle = "Dades Assegurança (Opcional)"
        section4.footerTitle = "Les dades de l'assegurança ajuden a calcular el cost diari del vehicle"
        
        //Impost municipal
        let section5 = FormSectionDescriptor()
        
        row = FormRowDescriptor(tag: Static.impostPagament, rowType: .SegmentedControl, title: "Pagament")
        row.configuration[FormRowDescriptor.Configuration.Options] = ["Anual", "Bimestral", "Trimestral"]
        row.configuration[FormRowDescriptor.Configuration.TitleFormatterClosure] = { value in
            switch( value ) {
            case "Anual":
                return "Anual"
            case "Bimestral":
                return "Bimestral"
            case "Trimestral":
                return "Trimestral"
            default:
                return nil
            }
            } as TitleFormatterClosure
        section5.addRow(row)
        
        row = FormRowDescriptor(tag: Static.impostMesPagament, rowType: .Picker, title: "Primer mes de pagament")
        row.configuration[FormRowDescriptor.Configuration.Options] = ["Gener", "Febrer", "Març", "Abril", "Maig", "Juny", "Juliol", "Agost", "Setembre", "Octubre", "Novembre", "Desembre"]
        row.configuration[FormRowDescriptor.Configuration.TitleFormatterClosure] = { value in
            switch( value ) {
            case "Gener":
                return "Gener"
            case "Febrer":
                return "Febrer"
            case "Març":
                return "Març"
            case "Abril":
                return "Abril"
            case "Maig":
                return "Maig"
            case "Juny":
                return "Juny"
            case "Juliol":
                return "Juliol"
            case "Agost":
                return "Agost"
            case "Setembre":
                return "Setembre"
            case "Octubre":
                return "Octubre"
            case "Novembre":
                return "Novembre"
            case "Desembre":
                return "Desembre"
            default:
                return nil
            }
            } as TitleFormatterClosure
        row.value = ""
        section5.addRow(row)
        
        row = FormRowDescriptor(tag: Static.impostPreu, rowType: .Decimal, title: "Preu")
        row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.placeholder" : "e.x. 235,04", "textField.textAlignment" : NSTextAlignment.Right.rawValue]
        section5.addRow(row)
        
        section5.headerTitle = "Impost municipal (Opcional)"
        section5.footerTitle = "Les dades de l'impost municipal ajuden a calcular el cost diari del vehicle"
        
        //Altres dades del vehicle
        let section6 = FormSectionDescriptor()
        
        row = FormRowDescriptor(tag: Static.marca, rowType: .Name, title: "Marca")
        row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.placeholder" : "e.x. Renault", "textField.textAlignment" : NSTextAlignment.Right.rawValue]
        section6.addRow(row)
        
        row = FormRowDescriptor(tag: Static.model, rowType: .Name, title: "Model")
        row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.placeholder" : "e.x. Megane", "textField.textAlignment" : NSTextAlignment.Right.rawValue]
        section6.addRow(row)
        
        row = FormRowDescriptor(tag: Static.matricula, rowType: .Name, title: "Matrícula")
        row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.placeholder" : "e.x. 4856 DDE", "textField.textAlignment" : NSTextAlignment.Right.rawValue]
        section6.addRow(row)
        
        row = FormRowDescriptor(tag: Static.portes, rowType: .Stepper, title: "Nº de portes")
        row.configuration[FormRowDescriptor.Configuration.MaximumValue] = 8.0
        row.configuration[FormRowDescriptor.Configuration.MinimumValue] = 0.0
        row.configuration[FormRowDescriptor.Configuration.Steps] = 1.0
        section6.addRow(row)
        
        section6.headerTitle = "Altres dades (Opcional)"
        

        /*
        let section8 = FormSectionDescriptor()
        
        row = FormRowDescriptor(tag: Static.button, rowType: .Button, title: "Dismiss")
        row.configuration[FormRowDescriptor.Configuration.DidSelectClosure] = {
            self.view.endEditing(true)
            } as DidSelectClosure
        section8.addRow(row)
        
        form.sections = [section1, section2, section3, section4, section5, section6, section7, section8]
        */
        
        form.sections = [section1, section2, section3, section4, section5, section6]
        
        self.form = form
    }
    
    /// MARK: Actions
    func desa(_: UIBarButtonItem!) {
        let dades = form.formValues()
        let valid = self.validarFormulari(dades)
        if valid == true {
            //Crear nou vehicle
            let appDelegate = (UIApplication.sharedApplication().delegate as! AppDelegate)
            let managedObjectContext = appDelegate.managedObjectContext
            
            //Comprovem que no existeixi un vehicle amb el mateix nom
            let vehicleRequest = NSFetchRequest(entityName: "Vehicle")
            vehicleRequest.predicate = NSPredicate(format: "nom = %@", dades["foto"]?[0] as! String)
            vehicleRequest.fetchLimit = 1
            if let test = managedObjectContext!.executeFetchRequest(vehicleRequest, error: nil)?.first as? Vehicle {
                let alert: UIAlertView = UIAlertView(title: "Error", message: "El vehicle ja exiteix", delegate: nil, cancelButtonTitle: "Dacrod")
                alert.show()
            } else {
                
                //No exiteix el vehicle, per tant el podem crear
                let vehicleNou = NSEntityDescription.insertNewObjectForEntityForName("Vehicle", inManagedObjectContext: managedObjectContext!) as! Vehicle
                
                omplirVehicle(vehicleNou, dades: dades, managedObjectContext: managedObjectContext)
                
                //guardar i Tancar la pantalla
                appDelegate.saveContext()
                dismissViewControllerAnimated(true, completion: nil)
            }
        }
    }
    
    func cancelar(_: UIBarButtonItem!) {
        dismissViewControllerAnimated(true, completion: nil)
    }
    
    func desa_canvis(_: UIBarButtonItem!) {
        let dades = form.formValues()
        let valid = self.validarFormulari(dades)
        if valid == true {
            //Editar dades vehicle
            let appDelegate = (UIApplication.sharedApplication().delegate as! AppDelegate)
            let managedObjectContext = appDelegate.managedObjectContext
            
            var comprovacioExtra = true
            
            if (dades["foto"]?[0] as! String) != vehicle.nom {
                // Comprovem que no hi hagi cap altre vehicle amb el mateix nom
                let vehicleRequest = NSFetchRequest(entityName: "Vehicle")
                vehicleRequest.predicate = NSPredicate(format: "nom = %@", dades["foto"]?[0] as! String)
                vehicleRequest.fetchLimit = 1
                if let test = managedObjectContext!.executeFetchRequest(vehicleRequest, error: nil)?.first as? Vehicle {
                    let alert: UIAlertView = UIAlertView(title: "Error", message: "Ja existeix un vehicle amb aquest nom", delegate: nil, cancelButtonTitle: "Dacrod")
                    alert.show()
                    comprovacioExtra = false
                }
            }
            
            if comprovacioExtra {
                //Obtenim el vehicle on guardar els canvis
                let vehicleRequest = NSFetchRequest(entityName: "Vehicle")
                vehicleRequest.predicate = NSPredicate(format: "nom = %@", vehicle.nom)
                vehicleRequest.fetchLimit = 1
                if let vehicleEditat = managedObjectContext!.executeFetchRequest(vehicleRequest, error: nil)?.first as? Vehicle {
                    //tenim el vehicle, cal editar-ne els canvis i guardar
                    omplirVehicle(vehicleEditat, dades: dades, managedObjectContext: managedObjectContext)
                
                    //guardar
                    appDelegate.saveContext()
                    self.view.endEditing(true)
                    if let tabBar = self.tabBarController as? TabBarController {

                        let vehicleRequest = NSFetchRequest(entityName: "Vehicle")
                        vehicleRequest.predicate = NSPredicate(format: "nom = %@", vehicleEditat.nom)
                        vehicle = managedObjectContext!.executeFetchRequest(vehicleRequest, error: nil)!.first as! Vehicle
                        tabBar.title = vehicle.nom
                    }
                   
                } else {
                    let alert: UIAlertView = UIAlertView(title: "Error", message: "No s'ha pogut guardar el vehicle", delegate: nil, cancelButtonTitle: "Dacrod")
                    alert.show()
                }
            }
        }
    }
    
    func desfer_canvis(_: UIBarButtonItem!) {
        emplenarForm()
        self.tableView.reloadData()
    }
    
    func emplenarForm(){
        
        if vehicle != nil {
            //Secció nom de vehicle i foto
            form.sections[0].rows[0].value = vehicle.nom
            form.sections[0].rows[0].configuration[FormRowDescriptor.Configuration.Image] = UIImage(data: vehicle.foto)
            
            //Secció Unitats i carburants
            form.sections[1].rows[0].value = Moneda(string: vehicle.moneda).simbol
            form.sections[1].rows[1].value = Distancia(string: vehicle.unitatKm).unitatMesura
            var carburants: NSMutableArray = []
            for carburant in vehicle.carburants {
                carburants.addObject((carburant as! Carburant).nom)
            }
            form.sections[1].rows[2].value = carburants as NSArray
            
            //Secció Compra de vehicle
            form.sections[2].rows[0].value = vehicle.obtenirDataCompra()
            form.sections[2].rows[1].value = vehicle.obtenirDataAdquisicio()
            form.sections[2].rows[2].value = (vehicle.preu as? Double)?.format(".2")
            
            //Secció Assegurança
            form.sections[3].rows[0].value = vehicle.seguro?.asseguradora
            form.sections[3].rows[1].value = vehicle.seguro?.pagament
            form.sections[3].rows[2].value = vehicle.seguro?.mesPagament
            form.sections[3].rows[3].value = (vehicle.seguro?.preu as? Double)?.format(".2")
            form.sections[3].rows[4].value = (vehicle.seguro?.franquicia as? Double)?.format(".2")
            form.sections[3].rows[5].value = vehicle.seguro?.tipus
            form.sections[3].rows[6].value = vehicle.seguro?.polissa
            form.sections[3].rows[7].value = vehicle.seguro?.notes
            
            //Secció Impost municipal
            form.sections[4].rows[0].value = vehicle.impostMunicipal?.pagament
            form.sections[4].rows[1].value = vehicle.impostMunicipal?.mesPagament
            form.sections[4].rows[2].value = (vehicle.impostMunicipal?.preu as? Double)?.format(".2")
            
            //Secció altres dades
            form.sections[5].rows[0].value = vehicle.marca
            form.sections[5].rows[1].value = vehicle.model
            form.sections[5].rows[2].value = vehicle.matricula
            form.sections[5].rows[3].value = vehicle.portes
        }
    }
    
    func validarFormulari(dades: NSDictionary) -> Bool{
        var missatge = ""
        if (dades["foto"]?[0] as? String ?? "") != "" {
            //Tenim un nom de vehicle introduit
            
            //Validar que hi ha almenys un carburant seleccionat
            if form.sections[1].rows[2].value == nil {
                missatge = "Cal indicar els carburants a usar"
            }
            
            //Validar si el formulari de l'assegurança està buit o ben emplenat (8 camps a comprovar)
            if missatge == "" {
                let asseguradora = (dades["asseguradora"] as? String) ?? ""
                let asseguradoraTipusPagament = (dades["asseguradoraTipusPagament"] as? String) ?? ""
                let asseguradoraPreu = (dades["asseguradoraPreu"] as? String) ?? ""
                let asseguradoraPolissa = (dades["asseguradoraPolissa"] as? String) ?? ""
                let asseguradoraFranquicia = (dades["asseguradoraFranquicia"] as? String) ?? ""
                let asseguradoraTipus = (dades["asseguradoraTipus"] as? String) ?? ""
                let asseguradoraNotes = (dades["asseguradoraNotes"] as? String) ?? ""
                let asseguradoraMesPagament = (dades["asseguradoraMesPagament"] as? String) ?? ""
                if asseguradora != "" || asseguradoraTipusPagament != "" || asseguradoraPreu != "" || asseguradoraPolissa != "" || asseguradoraFranquicia != "" || asseguradoraTipus != "" || asseguradoraNotes != "" || asseguradoraMesPagament != "" {
                    //Hi ha alguna dada plena, comprovar que les mínimes requerides estàn plenes
                    if asseguradora == "" || asseguradoraTipusPagament == "" || asseguradoraPreu == "" || asseguradoraMesPagament == ""{
                        //Falten dades imprescindibles
                        missatge = "Falten dades de l'assegurança, com a mínim cal indicar: Nom de l'asseguradora, freqüència de pagamet, el primer més de l'any de pagament i el preu de l'assegurança."
                    }
                }
            }
            
            //Validar si el formulari de l'impost municipal és correcte en cas de que estigui emplenat
            if missatge == "" {
                let impostPagament = (dades["impostPagament"] as? String) ?? ""
                let impostPreu = (dades["impostPreu"] as? String) ?? ""
                let impostMesPagament = (dades["impostMesPagament"] as? String) ?? ""
                
                let tots = impostPagament + impostPreu + impostMesPagament
                
                if tots != "" && (impostPagament == "" || impostPreu == "" || impostMesPagament == "") {
                    missatge = "Falen dades sobre l'impost municipal, tots els camps són obligatoris"
                }
            }
            
        } else {
            missatge = "Cal introduir com a mínim un nom de vehicle"
        }
        
        if missatge != ""{
            let alert: UIAlertView = UIAlertView(title: "Error", message: missatge, delegate: nil, cancelButtonTitle: "Dacrod")
            alert.show()
            return false
        } else {
            return true
        }
    }
    
    func omplirVehicle(vehicleNou: Vehicle, dades: NSDictionary, managedObjectContext: NSManagedObjectContext?){
        
        //Dades bàsiques del vehicle
        vehicleNou.nom = dades["foto"]?[0] as! String
        let imageData = dades["foto"]?[1] as! UIImage
        vehicleNou.foto = UIImageJPEGRepresentation(imageData, 0.8)
        
        //Tipus d'unitats i carburants
        vehicleNou.moneda = Moneda(string: form.sections[1].rows[0].value as! String).simbol
        vehicleNou.unitatKm = Distancia(string: form.sections[1].rows[1].value as! String).unitatMesura
        
        var carburantsVehicle = vehicleNou.mutableSetValueForKey("carburants")
        carburantsVehicle.removeAllObjects() //Ens assegurem d'eliminar els carburants antics
        let carburants = form.sections[1].rows[2].value as! NSArray
        for carburantNou in carburants {
            let carburantRequest = NSFetchRequest(entityName: "Carburant")
            carburantRequest.predicate = NSPredicate(format: "nom = %@", carburantNou as! String)
            carburantRequest.fetchLimit = 1
            let carburant = managedObjectContext!.executeFetchRequest(carburantRequest, error: nil)?.first as! Carburant
            carburantsVehicle.addObject(carburant)
        }
        vehicleNou.carburants = carburantsVehicle
        
        //Informació de compra del vehicle
        vehicleNou.dataCompra = form.sections[2].rows[0].value as? NSDate
        vehicleNou.dataAdquisicio = form.sections[2].rows[1].value as? NSDate
        vehicleNou.preu = (form.sections[2].rows[2].value as? NSString)?.toDoubleValue()
        
        //Informació de l'assegurança
        if let seguroAntic = vehicleNou.seguro {
            managedObjectContext!.deleteObject(seguroAntic)
        }
        if let test = form.sections[3].rows[0].value as? String {
            let seguro = NSEntityDescription.insertNewObjectForEntityForName("Seguro", inManagedObjectContext: managedObjectContext!) as! Seguro
            seguro.asseguradora = form.sections[3].rows[0].value as! String
            seguro.pagament = form.sections[3].rows[1].value as! String
            seguro.mesPagament = form.sections[3].rows[2].value as! String
            seguro.preu = (form.sections[3].rows[3].value as! NSString).toDoubleValue()!
            seguro.franquicia = (form.sections[3].rows[4].value as? NSString)?.toDoubleValue()
            seguro.tipus = form.sections[3].rows[5].value as? String
            seguro.polissa = form.sections[3].rows[6].value as? String
            seguro.notes = form.sections[3].rows[7].value as? String
            
            vehicleNou.seguro = seguro
        }
        
        //Informació de l'Impost Municipal
        if let impostAntic = vehicleNou.impostMunicipal {
            managedObjectContext!.deleteObject(impostAntic)
        }
        if let test = form.sections[4].rows[0].value as? String {
            let impostMunicipal = NSEntityDescription.insertNewObjectForEntityForName("ImpostMunicipal", inManagedObjectContext: managedObjectContext!) as! ImpostMunicipal
            impostMunicipal.pagament = form.sections[4].rows[0].value as! String
            impostMunicipal.mesPagament = form.sections[4].rows[1].value as! String
            impostMunicipal.preu = (form.sections[4].rows[2].value as! NSString).toDoubleValue()!
            vehicleNou.impostMunicipal = impostMunicipal
        }
        
        //Informació adicional
        vehicleNou.marca = form.sections[5].rows[0].value as? String
        vehicleNou.model = form.sections[5].rows[1].value as? String
        vehicleNou.matricula  = form.sections[5].rows[2].value as? String
        vehicleNou.portes = (form.sections[5].rows[3].value as? NSString)?.toDoubleValue()
    }

}
