//
//  ViatgesViewController.swift
//  Concot
//
//  Created by Andreu Font Bardolet on 8/6/15.
//  Copyright (c) 2015 Andreu Font Bardolet. All rights reserved.
//

import UIKit
import SwiftForms

class ViatgesViewController: FormViewController {
    
    var vehicle: Vehicle!
    
    var distancia: Double = 100
    var persones: Double = 2

    override func viewDidLoad() {
        super.viewDidLoad()

        vehicle = (self.tabBarController as! TabBarController).vehicle
        
        self.omplirFormulari()
        
    }
    
    override func viewDidAppear(animated: Bool) {
        vehicle = (self.tabBarController as! TabBarController).vehicle
        self.omplirFormulari()
    }
    
    required init(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        self.loadForm()
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    private func loadForm() {
        //Creen el formulari buit, després en el ViewDidLoad es decidirà què s'hi posa
        let form = FormDescriptor()
        
        self.form = form
    }
    
    private func omplirFormulari() {
        var valor = "Desconegut"
        let seccio1 = FormSectionDescriptor()
        
        let stringDistancia = distancia.format(".0")
        let stringPersones = persones.format(".0")
        
        var row: FormRowDescriptor = FormRowDescriptor(tag: "distancia", rowType: .Number, title: "Distància prevista", placeholder: "100")
        row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
        row.value = stringDistancia
        seccio1.addRow(row)
        
        row = FormRowDescriptor(tag: "persones", rowType: .Number, title: "Número de persones", placeholder: "2")
        row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
        row.value = stringPersones
        seccio1.addRow(row)
        
        let seccio2 = FormSectionDescriptor()
        row = FormRowDescriptor(tag: "calcular", rowType: .Button, title: "Calcular")
        row.configuration[FormRowDescriptor.Configuration.DidSelectClosure] = {
            
            if row.value != nil {
                self.calcular()
            }
            } as DidSelectClosure
        seccio2.addRow(row)
        
        
        //Comprovar que tenim prou dades com per calular els costs
        if let costCarburantUnitatDistancia = vehicle.obtenirCostCarburantPerUnitatDistancia() {
            
            //Calculem els totals
            let costUnitatDistancia = vehicle.obtenirCostPerUnitatDistancia()!
            
            let seccio3 = FormSectionDescriptor()
            seccio3.headerTitle = "Cost total"
            
            row = FormRowDescriptor(tag: "totalPersona", rowType: .Label, title: "Total per persona")
            row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
            valor = ((costUnitatDistancia * distancia) / persones).format(".2")
            row.value = "\(valor) \(Moneda(string: vehicle.moneda).simbol)"
            seccio3.addRow(row)
            
            row = FormRowDescriptor(tag: "totalViatge", rowType: .Label, title: "Total del viatge")
            row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
            valor = (costUnitatDistancia * distancia).format(".2")
            row.value = "\(valor) \(Moneda(string: vehicle.moneda).simbol)"
            seccio3.addRow(row)
            
            row = FormRowDescriptor(tag: "costTotalPerDistancia", rowType: .Label, title: "Cost per \(Distancia(string: vehicle.unitatKm).nom.lowercaseString)")
            row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
            valor = (costUnitatDistancia).format(".3")
            row.value = "\(valor) \(Moneda(string: vehicle.moneda).simbol)"
            seccio3.addRow(row)
            
            row = FormRowDescriptor(tag: "costTotalPerDistanciaPersona", rowType: .Label, title: "Cost per \(Distancia(string: vehicle.unitatKm).nom.lowercaseString) i persona")
            row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
            valor = (costUnitatDistancia / persones).format(".3")
            row.value = "\(valor) \(Moneda(string: vehicle.moneda).simbol)"
            seccio3.addRow(row)
            
            let seccio4 = FormSectionDescriptor()
            seccio4.headerTitle = "Detalls"
            
            row = FormRowDescriptor(tag: "totalCarburant", rowType: .Label, title: "Total en \(vehicle.obtenirDarrerCarburant()!.nom.lowercaseString)")
            row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
            valor = (costCarburantUnitatDistancia * distancia).format(".3")
            row.value = "\(valor) \(Moneda(string: vehicle.moneda).simbol)"
            seccio4.addRow(row)
            
            row = FormRowDescriptor(tag: "carburantPerDistancia", rowType: .Label, title: "Cost \(vehicle.obtenirDarrerCarburant()!.nom.lowercaseString) per \(Distancia(string: vehicle.unitatKm).nom.lowercaseString)")
            row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
            valor = costCarburantUnitatDistancia.format(".3")
            row.value = "\(valor) \(Moneda(string: vehicle.moneda).simbol)"
            seccio4.addRow(row)
            
            if let costDesgastUnitatDistancia = vehicle.obtenirCostDesgastPerUnitatDistancia() {
                row = FormRowDescriptor(tag: "totalDesgast", rowType: .Label, title: "Total en desgast")
                row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
                valor = (costDesgastUnitatDistancia * distancia).format(".3")
                row.value = "\(valor) \(Moneda(string: vehicle.moneda).simbol)"
                seccio4.addRow(row)
                
                row = FormRowDescriptor(tag: "totalDesgast", rowType: .Label, title: "Desgast per \(Distancia(string: vehicle.unitatKm).nom.lowercaseString)")
                row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
                valor = (costDesgastUnitatDistancia).format(".3")
                row.value = "\(valor) \(Moneda(string: vehicle.moneda).simbol)"
                seccio4.addRow(row)
                
                let seccio5 = FormSectionDescriptor()
                seccio5.headerTitle = "Detall de desgast"
                
                let manteniments = vehicle.manteniment as! NSMutableSet
                for manteniment in manteniments {
                    if let costUnitatDistancia = (manteniment as! Manteniment).costPerUnitatDistancia {
                        let nom = (manteniment as! Manteniment).nom
                        row = FormRowDescriptor(tag: "desgast\(nom)", rowType: .Label, title: "Total \(nom.lowercaseString)")
                        row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
                        valor = (costUnitatDistancia * distancia).format(".3")
                        row.value = "\(valor) \(Moneda(string: vehicle.moneda).simbol)"
                        seccio5.addRow(row)
                        
                        row = FormRowDescriptor(tag: "totalDesgast\(nom)", rowType: .Label, title: "\(nom) per \(Distancia(string: vehicle.unitatKm).unitatMesura)")
                        row.configuration[FormRowDescriptor.Configuration.CellConfiguration] = ["textField.textAlignment" : NSTextAlignment.Right.rawValue]
                        valor = (costUnitatDistancia).format(".3")
                        row.value = "\(valor) \(Moneda(string: vehicle.moneda).simbol)"
                        seccio5.addRow(row)
                    }
                }
                
                form.sections = [seccio1,  seccio2, seccio3, seccio4, seccio5]
            } else {
                seccio4.footerTitle = "No hi ha prou dades per calcular els costs relacionats amb el desgast"
                form.sections = [seccio1,  seccio2, seccio3, seccio4]
            }
        } else {
            let seccio3 = FormSectionDescriptor()
            //Comprovar si falten emplenaments en general o del carburant actual
            if let carburantActual = vehicle.obtenirDarrerCarburant() {
                seccio3.footerTitle = "Ho hi ha prou dades per calcular, falten emplenaments complets de \(carburantActual.nom)"
            } else {
                seccio3.footerTitle = "Ho hi ha prou dades per calcular els costs de viatges, cal registrar com a mínim dos emplenaments complets del mateix tipus de carburant"
            }
            form.sections = [seccio1,  seccio2, seccio3]
        }
        
        self.tableView.reloadData()
    }
    
    private func calcular(){
        let dades = form.formValues()
        
        if let novaDistancia = dades["distancia"]?.doubleValue {
            if let novesPersones = dades["persones"]?.doubleValue {
                distancia = novaDistancia
                persones = novesPersones
                self.omplirFormulari()
            }
        }
    }

}
