//
//  FormFotoCell.swift
//  Concot
//
//  Created by Andreu Font Bardolet on 28/5/15.
//  Copyright (c) 2015 Andreu Font Bardolet. All rights reserved.
//

import UIKit
import MobileCoreServices

public class FormFotoCell: FormBaseCell, UIImagePickerControllerDelegate, UINavigationControllerDelegate {
    
    /// MARK: Cell views
    
    public let titleLabel = UILabel()
    public let foto = UIImageView()
    public let boto = UIButton.buttonWithType(UIButtonType.System) as! UIButton
    public let textField = UITextField()
    
    /// MARK: Properties
    
    private var customConstraints: [AnyObject]!
    
    /// MARK: Class Funcs
    public override class func formRowCellHeight() -> CGFloat {
        return 120.0
    }
    
    /// MARK: FormBaseCell
    
    public override func configure() {
        super.configure()
        
        afegirBarraMenuTeclat()
        
        selectionStyle = .None
        
        //titleLabel.setTranslatesAutoresizingMaskIntoConstraints(false)
        textField.setTranslatesAutoresizingMaskIntoConstraints(false)
        foto.setTranslatesAutoresizingMaskIntoConstraints(false)
        boto.setTranslatesAutoresizingMaskIntoConstraints(false)
        
        //titleLabel.font = UIFont.preferredFontForTextStyle(UIFontTextStyleBody)
        textField.font = UIFont.preferredFontForTextStyle(UIFontTextStyleBody)
        
        boto.titleLabel?.font = UIFont.preferredFontForTextStyle(UIFontTextStyleBody)
        boto.contentVerticalAlignment = UIControlContentVerticalAlignment.Bottom
        boto.contentHorizontalAlignment = UIControlContentHorizontalAlignment.Center
        
        
        /*foto.frame.size.width = CGFloat(83)
        foto.frame.size.height = CGFloat(83)
        foto.frame.origin.x = CGFloat(8)
        foto.frame.origin.y = CGFloat(8)*/
        
        //contentView.addSubview(titleLabel)
        contentView.addSubview(foto)
        contentView.addSubview(boto)
        contentView.addSubview(textField)
        
        //titleLabel.setContentHuggingPriority(500, forAxis: .Horizontal)
        //titleLabel.setContentCompressionResistancePriority(1000, forAxis: .Horizontal)
        
        contentView.addConstraint(NSLayoutConstraint(item: foto, attribute: .Top, relatedBy: .Equal, toItem: contentView, attribute: .Top, multiplier: 1.0, constant: 8.0))
        contentView.addConstraint(NSLayoutConstraint(item: textField, attribute: .CenterY, relatedBy: .Equal, toItem: foto, attribute: .CenterY, multiplier: 1.0, constant: 0.0))

        
        contentView.addConstraint(NSLayoutConstraint(item: foto, attribute: .Height, relatedBy: .Equal, toItem: nil, attribute: .Height, multiplier: 1.0, constant: 83.0))
        contentView.addConstraint(NSLayoutConstraint(item: foto, attribute: .Width, relatedBy: .Equal, toItem: nil, attribute: .Width, multiplier: 1.0, constant: 83.0))
        contentView.addConstraint(NSLayoutConstraint(item: foto, attribute: .Left, relatedBy: .Equal, toItem: contentView, attribute: .Left, multiplier: 1.0, constant: 8.0))
        contentView.addConstraint(NSLayoutConstraint(item: textField, attribute: .Leading, relatedBy: .Equal, toItem: foto, attribute: .Trailing, multiplier: 1.0, constant: 8.0))
        contentView.addConstraint(NSLayoutConstraint(item: textField, attribute: .Trailing, relatedBy: .Equal, toItem: contentView, attribute: .Trailing, multiplier: 1.0, constant: 8.0))
        contentView.addConstraint(NSLayoutConstraint(item: textField, attribute: .Height, relatedBy: .Equal, toItem: foto, attribute: .Height, multiplier: 1.0, constant: 0.0))
        
        
        contentView.addConstraint(NSLayoutConstraint(item: boto, attribute: .CenterX, relatedBy: .Equal, toItem: foto, attribute: .CenterX, multiplier: 1.0, constant: 0.0))
        contentView.addConstraint(NSLayoutConstraint(item: boto, attribute: .Top, relatedBy: .Equal, toItem: foto, attribute: .Top, multiplier: 1.0, constant: 0.0))
        contentView.addConstraint(NSLayoutConstraint(item: boto, attribute: .Bottom, relatedBy: .Equal, toItem: foto, attribute: .Bottom, multiplier: 1.0, constant: 28.0))
        contentView.addConstraint(NSLayoutConstraint(item: boto, attribute: .Width, relatedBy: .Equal, toItem: foto, attribute: .Width, multiplier: 1.0, constant: 0.0))
        
        foto.layer.cornerRadius = 41.5
        foto.clipsToBounds = true
        
        boto.addTarget(self, action: "fotoButton:", forControlEvents: UIControlEvents.TouchUpInside)
        textField.addTarget(self, action: "editingChanged:", forControlEvents: .EditingChanged)
    }
    
    public override func update() {
        super.update()
        
        if let showsInputToolbar = rowDescriptor.configuration[FormRowDescriptor.Configuration.ShowsInputToolbar] as? Bool {
            if showsInputToolbar && textField.inputAccessoryView == nil {
                textField.inputAccessoryView = inputAccesoryView()
            }
        }
        if let imatge = rowDescriptor.configuration[FormRowDescriptor.Configuration.Image] as? UIImage {
            foto.image = imatge
            boto.setTitle("Editar", forState: UIControlState.Normal)
        } else {
            foto.image = UIImage(named: "Placeholder")
            boto.setTitle("Afegir", forState: UIControlState.Normal)
        }
        rowDescriptor.image = foto.image //Evitar un bug...
        
        titleLabel.text = rowDescriptor.title
        textField.text = rowDescriptor.value as? String
        textField.placeholder = rowDescriptor.configuration[FormRowDescriptor.Configuration.Placeholder] as? String
        
        
        textField.secureTextEntry = false
        textField.clearButtonMode = .WhileEditing
        
        switch rowDescriptor.rowType {
        case .Text:
            textField.autocorrectionType = .Default
            textField.autocapitalizationType = .Sentences
            textField.keyboardType = .Default
        case .Number:
            textField.keyboardType = .NumberPad
        case .NumbersAndPunctuation:
            textField.keyboardType = .NumbersAndPunctuation
        case .Decimal:
            textField.keyboardType = .DecimalPad
        case .Name:
            textField.autocorrectionType = .No
            textField.autocapitalizationType = .Words
            textField.keyboardType = .Default
        case .Phone:
            textField.keyboardType = .PhonePad
        case .NamePhone:
            textField.autocorrectionType = .No
            textField.autocapitalizationType = .Words
            textField.keyboardType = .NamePhonePad
        case .URL:
            textField.autocorrectionType = .No
            textField.autocapitalizationType = .None
            textField.keyboardType = .URL
        case .Twitter:
            textField.autocorrectionType = .No
            textField.autocapitalizationType = .None
            textField.keyboardType = .Twitter
        case .Email:
            textField.autocorrectionType = .No
            textField.autocapitalizationType = .None
            textField.keyboardType = .EmailAddress
        case .ASCIICapable:
            textField.autocorrectionType = .No
            textField.autocapitalizationType = .None
            textField.keyboardType = .ASCIICapable
        case .Password:
            textField.secureTextEntry = true
            textField.clearsOnBeginEditing = false
        case .Foto:
            textField.autocorrectionType = .No
            textField.autocapitalizationType = .Sentences
            textField.keyboardType = .Default
        default:
            break
        }
    }
    
    public override func constraintsViews() -> [String : UIView] {
        var views = ["image" : foto, "textField" : textField]
        if self.imageView!.image != nil {
            views["imageView"] = imageView
        }
        return views
    }
    
    public override func defaultVisualConstraints() -> [String] {
        
        return ["H:|-16-[image]-[textField]-16-|"]
        /*if self.imageView!.image != nil {
            
            if titleLabel.text != nil && count(titleLabel.text!) > 0 {
                return ["H:[imageView]-[image]-[textField]-16-|"]
            }
            else {
                return ["H:[imageView]-[textField]-16-|"]
            }
        }
        else {
            if titleLabel.text != nil && count(titleLabel.text!) > 0 {
                return ["H:|-16-[image]-[textField]-16-|"]
            }
            else {
                return ["H:|-16-[textField]-16-|"]
            }
        }*/
    }
    
    public override func firstResponderElement() -> UIResponder? {
        return textField
    }
    
    public override class func formRowCanBecomeFirstResponder() -> Bool {
        return true
    }
    
    /// MARK: Actions
    
    internal func editingChanged(sender: UITextField) {
        let trimmedText = sender.text.stringByTrimmingCharactersInSet(NSCharacterSet.whitespaceAndNewlineCharacterSet())
        rowDescriptor.value = count(trimmedText) > 0 ? trimmedText : nil
    }
    
    // MARK: - UIImagePickerControllerDelefate
    public func imagePickerController(picker: UIImagePickerController, didFinishPickingMediaWithInfo info: [NSObject : AnyObject]) {
        if let image = info[UIImagePickerControllerEditedImage] as? UIImage {
            foto.image = image
            rowDescriptor.configuration[FormRowDescriptor.Configuration.Image] = image
            rowDescriptor.image = image
            boto.setTitle("Editar", forState: UIControlState.Normal)
            
        } else if let image = info[UIImagePickerControllerEditedImage] as? UIImage {
            foto.image = image
            rowDescriptor.configuration[FormRowDescriptor.Configuration.Image] = image
            rowDescriptor.image = image
            boto.setTitle("Editar", forState: UIControlState.Normal)
        }
        picker.dismissViewControllerAnimated(true, completion: nil)
    }
    
    internal func fotoButton(sender: UIButton) {
        let alertController = UIAlertController(title: nil, message: nil, preferredStyle: .ActionSheet)
        var topController = UIApplication.sharedApplication().keyWindow!.rootViewController as UIViewController!
        //var top = UIViewController()
        while ((topController.presentedViewController) != nil) {
            topController = topController.presentedViewController!;
        }
        // Crear accions dels butons.
        let calcelarAction = UIAlertAction(title: "Cancel·lar", style: .Cancel) { action in }
        alertController.addAction(calcelarAction)
        
        if UIImagePickerController.isSourceTypeAvailable(UIImagePickerControllerSourceType.Camera) {
            let ferFotoAction = UIAlertAction(title: "Fer foto", style: .Default) { action in
                var cameraController = UIImagePickerController()
                cameraController.delegate = self
                cameraController.sourceType = UIImagePickerControllerSourceType.Camera
                
                let mediaTypes:[AnyObject] = [kUTTypeImage]
                cameraController.mediaTypes = mediaTypes
                cameraController.allowsEditing = true
                topController.presentViewController(cameraController, animated: true, completion: nil)
            }
            alertController.addAction(ferFotoAction)
        }
        
        if UIImagePickerController.isSourceTypeAvailable(UIImagePickerControllerSourceType.PhotoLibrary) {
            let seleccionarFotoAction = UIAlertAction(title: "Seleccionar foto", style: .Default) { action in
                var photoLibraryController = UIImagePickerController()
                photoLibraryController.delegate = self
                photoLibraryController.sourceType = UIImagePickerControllerSourceType.PhotoLibrary
                
                let mediaTypes:[AnyObject] = [kUTTypeImage]
                photoLibraryController.mediaTypes = mediaTypes
                photoLibraryController.allowsEditing = true
                
                topController.presentViewController(photoLibraryController, animated: true, completion: nil)
            }
            alertController.addAction(seleccionarFotoAction)
        }
        topController.presentViewController(alertController, animated:true, completion:nil)
        //self.presentViewController(alertController, animated: true, completion: nil)
    }
    
    private func afegirBarraMenuTeclat(){
        var barraMenu: UIToolbar = UIToolbar(frame: CGRectMake(0, 0, 320, 50))
        barraMenu.barStyle = UIBarStyle.Default
        
        var flexSpace = UIBarButtonItem(barButtonSystemItem: UIBarButtonSystemItem.FlexibleSpace, target: nil, action: nil)
        var done: UIBarButtonItem = UIBarButtonItem(title: "Ok", style: UIBarButtonItemStyle.Done, target: self, action: Selector("okApretat"))
        
        var items = NSMutableArray()
        items.addObject(flexSpace)
        items.addObject(done)
        
        barraMenu.items = items as [AnyObject]
        barraMenu.sizeToFit()
        
        
        self.textField.inputAccessoryView = barraMenu
    }
    
    func okApretat(){
        self.textField.resignFirstResponder()
    }

}